from django.core.cache import cache
from django.contrib.sites.models import Site
from django.template import Library, Node, Variable
from django.template import TemplateSyntaxError, VariableDoesNotExist
from product.models import Category
from satchmo_utils.templatetags import get_filter_args

import logging
import re

log = logging.getLogger('shop.templatetags')

try:
    from xml.etree.ElementTree import Element, SubElement, tostring
except ImportError:
    from elementtree.ElementTree import Element, SubElement, tostring

register = Library()

def recurse_for_children(current_node, parent_node, active_cat, show_empty=True):
    child_count = current_node.child.active().count()

    if show_empty or child_count > 0 or current_node.product_set.count() > 0:
        li_id = 'category-%s' % current_node.id
        li_attrs = {'id': li_id }
        temp_parent = SubElement(parent_node, 'li', li_attrs)
        attrs = {'href': current_node.get_absolute_url()}
        link = SubElement(temp_parent, 'a', attrs)
        link.text = current_node.translated_name()

        if child_count > 0:
            new_parent = SubElement(temp_parent, 'ul')
            children = current_node.child.active()
            for child in children:
                recurse_for_children(child, new_parent, active_cat)

@register.simple_tag
def category_tree(id=None):
    """
    Creates an unnumbered list of the categories.

    Example::

        <ul>
            <li>Books
                <ul>
                <li>Science Fiction
                    <ul>
                    <li>Space stories</li>
                    <li>Robot stories</li>
                    </ul>
                </li>
                <li>Non-fiction</li>
                </ul>
        </ul>
    """
    active_cat = None
    if id:
        try:
            active_cat = Category.objects.active().get(id=id)
        except Category.DoesNotExist:
            active_cat = None
    # We call the category on every page so we will cache
    # The actual structure to save db hits
    current_site = Site.objects.get_current()
    cache_key = "cat-%s" % current_site.id
    existing_tree = cache.get(cache_key, None)
    if existing_tree is None:
        root = Element("ul")
        for cats in Category.objects.root_categories():
            recurse_for_children(cats, root, active_cat)
        existing_tree = root
        cache.set(cache_key, existing_tree)
    # If we have an active cat, search through and identify it
    # This search is less expensive than the multiple db calls
    if active_cat:
        active_cat_id = "category-%s" % active_cat.id
        for li in existing_tree.getiterator("li"):
            if li.attrib["id"] == active_cat_id:
                link = li.find("a")
                link.attrib["class"] = "current"
                break
    return tostring(existing_tree, 'utf-8')

class CategoryListNode(Node):
    """Template Node tag which pushes the category list into the context"""
    def __init__(self, slug, var, nodelist):
        self.var = var
        self.slug = slug
        self.nodelist = nodelist

    def render(self, context):

        if self.slug:
            try:
                cat = Category.objects.active().get(slug__iexact=self.slug.resolve(context))
                cats = cat.child.all()
            except (Category.DoesNotExist, VariableDoesNotExist):
                log.warn("No category found for slug: %s", self.slug)
                cats = []

        else:
            cats = Category.objects.root_categories()

        context[self.var] = cats

        context.push()
        context[self.var] = cats
        output = self.nodelist.render(context)
        context.pop()
        return output

@register.tag
def category_list(parser, token):
    """Push the category list into the context using the given variable name.

    Sample usage::

        {% category_list slug as var %}
        or
        {% category_list as var %}


    """
    args = token.split_contents()
    ct = len(args)
    if not ct in (3,4):
        raise TemplateSyntaxError("%r tag expecting '[slug] as varname', got: %s" % (args[0], args))

    if ct == 3:
        slug = None
        var = args[2]
    else:
        slug = Variable(args[1])
        var = args[3]

    nodelist = parser.parse(('endcategory_list',))
    parser.delete_first_token()

    return CategoryListNode(slug, var, nodelist)

class SetVariableInContextNode(Node):
    def __init__(self, var, val):
        self.var = var
        self.val = val

    def render(self, context):
        context[self.var] = self.val
        return ''

@register.tag
def categories_for_slugs(parser, token):
    """
    Usage: {% categories_for_slugs "slug[,slug...]" as varname %}

    Sets the variable *varname* in the context to a list of categories, given by
    the list of slugs.

    Useful if you want to specify a custom list of categories and override the
    default category listing from satchmo. 
    
    Example usage::

        {% categories_for_slug "hats,boots,accessories" as categories %}
        <ul>
            {% for child in categories.child.active %}
            <li><a href="{{ child.get_absolute_url }}">{{ child.translated_name }}</a></li>
            {% endfor %}
        </ul>
        
    """
    try:
        # Splitting by None == splitting by spaces.
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise TemplateSyntaxError, "%r tag requires arguments" \
              % token.contents.split()[0]

    m = re.search(r'"([^ "]+)" as (\w+)', arg)

    if not m:
        raise TemplateSyntaxError, "%r tag had invalid arguments" \
              % tag_name

    cat_slugs, var = m.groups()
    cats=[]

    for cat_slug in cat_slugs.split(','):
        try:
            cat = Category.objects.get(slug__iexact=cat_slug)
        except Category.DoesNotExist:
            log.warn("No category found for slug: %s", cat_slug)
            cat = None
        cats.append(cat)

    return SetVariableInContextNode(var, cats)

@register.filter
def product_category_siblings(product, args=""):
    args, kwargs = get_filter_args(args,
        keywords=('variations', 'include_self'),
        boolargs=('variations', 'include_self'),
        stripquotes=True)

    sibs = product.get_category.product_set.all().order_by('ordering', 'name')
    if not kwargs.get('variations', True):
        sibs = [sib for sib in sibs if not sib.has_variants]

    if not kwargs.get('include_self', True):
        sibs = [sib for sib in sibs if not sib == product]

    return sibs

class AllProductsForVariableSlugNode(Node):
    """
    Sets the variable *var* in the context to result of

      category.active_products(include_children=True)

    where category is the instance of Category with the slug specified in the
    context variable *slug_var*.
    """
    def __init__(self, slug_var, var):
        self.slug_var = slug_var
        self.var = var

    def render(self, context):
        try:
            slug = self.slug_var.resolve(context)
        except VariableDoesNotExist:
            log.error("The variable '%s' was not found in the context.", self.slug_var)
            return ''

        try:
            cat = Category.objects.active().get(slug__iexact=slug)
        except Category.DoesNotExist:
            log.error("No category found for slug: %s" % slug)
            return ''

        context[self.var] = cat.active_products(include_children=True)
        return ''

class AllProductsForSlugNode(Node):
    """
    Sets the variable *var* in the context to result of

      category.active_products(include_children=True)

    where category is the instance of Category with the slug *slug*.
    """
    def __init__(self, slug, var):
        self.slug = slug
        self.var = var

    def render(self, context):
        try:
            cat = Category.objects.active().get(slug__iexact=self.slug)
        except Category.DoesNotExist:
            log.error("No category found for slug: %s" % self.slug)
            return ''

        context[self.var] = cat.active_products(include_children=True)
        return ''

class AllProductsNode(Node):
    """
    Sets the variable *var* in the context to result of

      category.active_products(include_children=True)

    where category is the variable *category* in the context.
    """
    def __init__(self, var):
        self.var = var

    def render(self, context):
        cat = context.get('category')

        if not cat:
            log.error("The variable 'category' was not found in the context.")

        context[self.var] = cat.active_products(include_children=True)
        return ''

@register.tag
def all_products_for_category(parser, token):
    """
    Usage:
     1. {% all_products_for_category as varname %}
     2. {% all_products_for_category for slug_var as varname %}
     3. {% all_products_for_category for "slug" as varname %}

    Sets the variable *varname* in the context to a list of all products that
    are active in *category*, and is equivalent to the result of:

      category.active_products(include_children=True)

    where *category* is:
     1. the 'category' variable in the context, for usage 1.
     2. the instance of Category with the slug in the context variable
        *slug_var*, for usage 2.
     3. the instance of Category with the slug *slug*, for usage 3.
    """
    try:
        # Splitting by None == splitting by spaces.
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise TemplateSyntaxError, "%r tag requires arguments" \
              % token.contents.split()[0]

    m = re.search(r'(.*?)as (\w+)$', arg)

    # First, get the varname - the easiest
    if not m:
        raise TemplateSyntaxError, "Variable name was not specified for %r tag" \
              % tag_name

    arg, var  = m.groups()

    # Now, try and determine usage case the user wants
    if not arg:
        # We're of the first case.
        return AllProductsNode(var)

    m = re.search(r'^for (.+?)$', arg.strip())

    if not m:
        raise TemplateSyntaxError, "Invalid arguments for %r tag" \
              % tag_name

    arg = m.group(1)

    if arg[0] == '"' and arg[-1] == '"':
        # We're of the third case.
        cat_var = arg[1:-1]
        return AllProductsForSlugNode(arg[1:-1], var)
    elif arg:
        # We're of the second case.
        return AllProductsForVariableSlugNode(Variable(arg), var)

    raise TemplateSyntaxError, "Invalid arguments for %r tag" \
          % tag_name



from django.conf import settings as djangosettings
from django.contrib.sites.models import Site
from django.http import Http404
from django.test import TestCase
import keyedcache
from livesettings import *
import re
import time
import logging
log = logging.getLogger('test');

class ConfigurationFunctionTest(TestCase):
    
    def testSetSingleConfigItem(self):
        value = IntegerValue(BASE_GROUP, 'SingleItem')
        config_register(value)
        self.assert_(config_exists(BASE_GROUP, 'SingleItem'))
        
    def testSetTwoConfigItems(self):
        s = [IntegerValue(BASE_GROUP, 'testTwoA'), StringValue(BASE_GROUP, 'testTwoB')]
        config_register_list(*s)
        
        self.assert_(config_exists(BASE_GROUP, 'testTwoA'))
        self.assert_(config_exists(BASE_GROUP, 'testTwoB'))
        
    def testSetGroup(self):
        g1 = ConfigurationGroup('test1','test1')
        value = IntegerValue(g1, 'SingleGroupedItem')
        config_register(value)
        self.assertFalse(config_exists(BASE_GROUP, 'SingleGroupedItem'))
        self.assert_(config_exists(g1, 'SingleGroupedItem'))


class ConfigurationTestSettings(TestCase):
    
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        g = ConfigurationGroup('test2','test2')
        self.g = g
        config_register(StringValue(g, 's1'))
        config_register(IntegerValue(g, 's2', default=10))
        config_register(IntegerValue(g, 's3', default=10))
        
    def testSetSetting(self):
        c = config_get('test2', 's1')
        c.update('test')
        
        self.assertEqual(c.value, 'test')
        self.assertEqual(c.setting.value, 'test')
        
    def testSettingDefault(self):
        c = config_get('test2', 's2')        
        self.assertEqual(c.value, 10)
        
    def testSetAndReset(self):
        """Test setting one value and then updating"""
        c = config_get('test2', 's1')
        c.update('test1')

        self.assertEqual(c.value, 'test1')
        
        # should be true, since it is an update
        self.assert_(c.update('test2'))
        self.assertEqual(c.value, 'test2')
        
    def testTwice(self):
        """Config items should respond False to duplicate requests to update."""
        
        c = config_get('test2', 's1')
        c.update('test1')
        
        self.assertFalse(c.update('test1'))
        
        
    def testDeletesDefault(self):
        c = config_get('test2', 's3')
        # false because it isn't saving a default value
        self.assertFalse(c.update(10))
                    
        self.assert_(c.update(20))
        self.assertEqual(c.value, 20)
        try:
            s = c.setting
        except SettingNotSet:
            self.fail("Should have a setting now")
            
        # now delete and go back to no setting by setting the default
        self.assert_(c.update(10))
        self.assertEqual(c.value, 10)
        
        try:
            s = c.setting
            self.fail('Should throw SettingNotSet')
        except SettingNotSet:
            pass
        

class ConfigTestDotAccess(TestCase):
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g = ConfigurationGroup('test3','test3')
        self.g = g
        c1 = config_register(BooleanValue(g, 's1', default=True))
        c2 = config_register(IntegerValue(g, 's2', default=10))
        c2.update(100)
        
    def testDotAccess(self):
        self.assert_(ConfigurationSettings().test3.s1.value)
        self.assertEqual(ConfigurationSettings().test3.s2.value, 100)
        
    def testSettingProperty(self):
        c = config_get('test3','s2')
        s = c.setting
        self.assert_(s.value, 100)

class ConfigTestModuleValue(TestCase):
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g = ConfigurationGroup('modules','module test')
        self.g = g
        self.c = config_register(ModuleValue(g, 'test'))
    
    def testModule(self):
        c = config_get('modules', 'test')
        c.update('satchmo_store')
        
        self.assert_(hasattr(self.c.value, 'get_version'))
        
class ConfigTestSortOrder(TestCase):
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g1 = ConfigurationGroup('group1', 'Group 1', ordering=-1001)
        g2 = ConfigurationGroup('group2', 'Group 2', ordering=-1002)
        g3 = ConfigurationGroup('group3', 'Group 3', ordering=-1003)
        
        self.g1 = g1
        self.g2 = g2
        self.g3 = g3
        
        self.g1c1 = config_register(IntegerValue(g1, 'c1'))
        self.g1c2 = config_register(IntegerValue(g1, 'c2'))
        self.g1c3 = config_register(IntegerValue(g1, 'c3'))
        
        self.g2c1 = config_register(IntegerValue(g2, 'c1'))
        self.g2c2 = config_register(IntegerValue(g2, 'c2'))
        self.g2c3 = config_register(IntegerValue(g2, 'c3'))
    
        self.g3c1 = config_register(IntegerValue(g3, 'c1'))
        self.g3c2 = config_register(IntegerValue(g3, 'c2'))
        self.g3c3 = config_register(IntegerValue(g3, 'c3'))
    
    def testGroupOrdering(self):
        mgr = ConfigurationSettings()
        self.assertEqual(mgr[2].key, self.g1.key)
        self.assertEqual(mgr[1].key, self.g2.key)
        self.assertEqual(mgr[0].key, self.g3.key)
        
        
class TestMultipleValues(TestCase):
    
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g1 = ConfigurationGroup('m1', 'Multiple Group 1', ordering=1000)
        self.g1 = g1
         
        self.g1c1 = config_register(MultipleStringValue(g1,
            'c1',
            choices=((1,'one'),(2,'two'),(3,'three'))))
            
    def testSave(self):
        
        c = config_get('m1','c1')
        c.update([1,2])        
        self.assertEqual(c.value, [1,2])
        
    def testAddChoice(self):
        
        config_add_choice('m1','c1',(4, 'four'))
        c = config_get('m1','c1')
        self.assertEqual(c.choices, ((1,'one'),(2,'two'),(3,'three'),(4,'four')))
        
    def testChoiceValues(self):
        self.g1c1.update([1,2])
        
        self.assertEqual(self.g1c1.value, [1,2])
        self.assertEqual(self.g1c1.choice_values, [(1, 'one'),(2, 'two')])
        
        choices = config_choice_values('m1', 'c1')
        self.assertEqual(choices, [(1, 'one'),(2, 'two')])
            
class TestMultipleValuesWithDefault(TestCase):

    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()

        g1 = ConfigurationGroup('mv2', 'Multiple Group 2', ordering=1000)
        self.g1 = g1

        self.g1c1 = config_register(MultipleStringValue(g1,
            'c1',
            choices=((1,'one'),(2,'two'),(3,'three')),
            default=[1,2]))

    def testDefault(self):

        c = config_get('mv2','c1')
        self.assertEqual(c.value, [1,2])
        
        c.update([1,2,3])        
        self.assertEqual(c.value, [1,2,3])

class ConfigTestChoices(TestCase):

    def testAddPreregisteredChoice(self):
        """Test that we can register choices before the config is actually set up."""
        config_add_choice('ctg1', 'c1', ('a', 'Item A'))
        config_add_choice('ctg1', 'c1', ('b', 'Item B'))
        config_add_choice('ctg1', 'c1', ('c', 'Item C'))
        
        g1 = ConfigurationGroup('ctg1', 'Choice 1', ordering=1000)
        config_register(StringValue(g1, 'c1'))
            
        c = config_get('ctg1','c1')
        
        self.assertEqual(c.choices, [('a','Item A'), ('b','Item B'), ('c','Item C')])
        
        
class ConfigTestRequires(TestCase):

    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g1 = ConfigurationGroup('req1', 'Requirements 1', ordering=1000)

        self.g1 = g1

        bool1 = config_register(BooleanValue(g1, 'bool1', default=False, ordering=1))
        bool2 = config_register(BooleanValue(g1, 'bool2', ordering=2))
        
        self.g1c1 = config_register(IntegerValue(g1, 'c1', requires=bool1, ordering=3))

        self.g1c2 = config_register(IntegerValue(g1, 'c2', requires=bool2, ordering=4))
        self.g1c3 = config_register(IntegerValue(g1, 'c3', ordering=5))
        
        bool2.update(True)        
        
    def testSimpleRequires(self):
        
        v = config_value('req1', 'bool2')
        self.assertTrue(v)
        
        keys = [cfg.key for cfg in self.g1]
        self.assertEqual(keys, ['bool1', 'bool2', 'c2','c3'])
        
        c = config_get('req1','bool1')
        c.update(True)
        
        keys = [cfg.key for cfg in self.g1]
        self.assertEqual(keys, ['bool1', 'bool2', 'c1', 'c2', 'c3'])
        
class ConfigTestRequiresChoices(TestCase):

    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
        
        g1 = ConfigurationGroup('req2', 'Requirements 2', ordering=1000)

        self.g1 = g1

        choices1 = config_register(MultipleStringValue(BASE_GROUP, 'rc1', ordering=1))

        self.g1c1 = config_register(IntegerValue(g1, 'c1', requires=choices1, ordering=3))
        self.g1c2 = config_register(IntegerValue(g1, 'c2', requires=choices1, ordering=4))
        self.g1c3 = config_register(IntegerValue(g1, 'c3', ordering=5))

        choices1.update('c1')
        
        g2 = ConfigurationGroup('req3', 'Requirements 3', ordering=1000)

        self.g2 = g2
        
        choices2 = config_register(StringValue(BASE_GROUP, 'choices2', ordering=1))

        self.g2c1 = config_register(IntegerValue(g2, 'c1', requires=choices2, ordering=3))
        self.g2c2 = config_register(IntegerValue(g2, 'c2', requires=choices2, ordering=4))
        self.g2c3 = config_register(IntegerValue(g2, 'c3', requires=choices2, ordering=5))
        
        choices2.update('c1')

    def testSimpleRequiresChoices(self):

        v = config_value('BASE', 'rc1')
        self.assertEquals(v, ['c1'])

        g = config_get_group('req2')
        keys = [cfg.key for cfg in g]
        self.assertEqual(keys, ['c1','c3'])

        c = config_get('BASE', 'rc1')
        c.update(['c1','c2'])

        g = config_get_group('req2')
        keys = [cfg.key for cfg in g]
        self.assertEqual(keys, ['c1', 'c2', 'c3'])
        
    def testRequiresSingleValue(self):
        v = config_value('BASE', 'choices2')
        self.assertEquals(v, 'c1')

        keys = [cfg.key for cfg in self.g2]
        self.assertEqual(keys, ['c1'])

        c = config_get('BASE', 'choices2')
        c.update('c2')

        keys = [cfg.key for cfg in self.g2]
        self.assertEqual(keys, ['c2'])
        
class ConfigTestRequiresValue(TestCase):
    
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()

        g1 = ConfigurationGroup('reqval', 'Requirements 3', ordering=1000)

        self.g1 = g1

        choices1 = config_register(MultipleStringValue(BASE_GROUP, 'valchoices', ordering=1))

        self.g1c1 = config_register(IntegerValue(g1, 'c1', requires=choices1, requiresvalue='foo', ordering=3))
        self.g1c2 = config_register(IntegerValue(g1, 'c2', requires=choices1, requiresvalue='bar', ordering=4))
        self.g1c3 = config_register(IntegerValue(g1, 'c3', ordering=5))

        choices1.update('foo')
        
        g2 = ConfigurationGroup('reqval2', 'Requirements 4', ordering=1000)

        self.g2 = g2
        
        choices2 = config_register(StringValue(BASE_GROUP, 'valchoices2', ordering=1,
            choices=(('a','test a'),('b', 'test b'),('c', 'test c'))))

        self.g2c1 = config_register(IntegerValue(g2, 'c1', requires=choices2, requiresvalue='a', ordering=3))
        self.g2c2 = config_register(IntegerValue(g2, 'c2', requires=choices2, requiresvalue='b', ordering=4))
        self.g2c3 = config_register(IntegerValue(g2, 'c3', requires=choices2, requiresvalue='c', ordering=5))
        
        choices2.update('a')
    
    def testRequiresValue(self):
        v = config_value('BASE', 'valchoices')
        self.assertEquals(v, ['foo'])

        g = config_get_group('reqval')

        keys = [cfg.key for cfg in g]
        self.assertEqual(keys, ['c1','c3'])

        c = config_get('BASE', 'valchoices')
        c.update(['foo','bar'])

        g = config_get_group('reqval')
        keys = [cfg.key for cfg in g]
        self.assertEqual(keys, ['c1', 'c2', 'c3'])
        
    def testRequiresSingleValue(self):
        v = config_value('BASE', 'valchoices2')
        self.assertEquals(v, 'a')
        
        keys = [cfg.key for cfg in self.g2]
        self.assertEqual(keys, ['c1'])
        
        c = config_get('BASE', 'valchoices2')
        c.update('b')

        keys = [cfg.key for cfg in self.g2]
        self.assertEqual(keys, ['c2'])

class ConfigTestGroupRequires(TestCase):
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()

        choices1 = config_register(MultipleStringValue(BASE_GROUP, 'groupchoice', ordering=1))
        choices2 = config_register(MultipleStringValue(BASE_GROUP, 'groupchoice2', ordering=1))

        g1 = ConfigurationGroup('groupreq', 'Requirements 4', ordering=1000, requires=choices1)
        self.g1 = g1
        
        self.g1c1 = config_register(IntegerValue(g1, 'c1', ordering=3))
        self.g1c2 = config_register(IntegerValue(g1, 'c2', requires=choices2, requiresvalue='bar', ordering=4))
        self.g1c3 = config_register(IntegerValue(g1, 'c3', ordering=5))
    
    def testRequiresValue(self):
        c = config_get('BASE', 'groupchoice')
        self.assertEquals(c.value, [])

        keys = [cfg.key for cfg in self.g1]
        self.assertEqual(keys, [])

        c2 = config_get('BASE', 'groupchoice2')
        c2.update('bar')

        keys = [cfg.key for cfg in self.g1]
        self.assertEqual(keys, ['c2'])

        c.update(['groupreq'])

        keys = [cfg.key for cfg in self.g1]
        self.assertEqual(keys, ['c1', 'c2', 'c3'])

class ConfigCollectGroup(TestCase):
    def setUp(self):
        keyedcache.cache_delete()
        choices = config_register(MultipleStringValue(BASE_GROUP, 'collect', ordering=1))
        self.choices = choices
        
        g1 = ConfigurationGroup('coll1', 'Collection 1')
        g2 = ConfigurationGroup('coll2', 'Collection 2')
        g3 = ConfigurationGroup('coll3', 'Collection 3')
        
        g1c1 = config_register(StringValue(g1, 'test'))
        g1c2 = config_register(StringValue(g1, 'test1'))
        g2c1 = config_register(StringValue(g2, 'test'))
        g3c1 = config_register(StringValue(g3, 'test'))
        
        g1c1.update('set a')
        g1c2.update('set b')
        g2c1.update('set a')
        g3c1.update('set d')
        
        choices.update(['coll1','coll3'])
        
    def testCollectSimple(self):
        v = config_collect_values('BASE', 'collect', 'test')
        
        self.assertEqual(v, ['set a', 'set d'])
        
    def testCollectUnique(self):
        self.choices.update(['coll1','coll2','coll3'])
        
        v = config_collect_values('BASE', 'collect', 'test', unique=False)
        
        self.assertEqual(v, ['set a', 'set a', 'set d'])
        
        v = config_collect_values('BASE', 'collect', 'test', unique=True)
        
        self.assertEqual(v, ['set a', 'set d'])
        
class LongSettingTest(TestCase):
    def setUp(self):
        keyedcache.cache_delete()
        wide = config_register(LongStringValue(BASE_GROUP, 'LONG', ordering=1, default="woot"))
        self.wide = wide
        self.wide.update('*' * 1000)
        
    def testLongStorage(self):
        w = config_value('BASE', 'LONG')
        self.assertEqual(len(w), 1000)
        self.assertEqual(w, '*'*1000)
        
    def testShortInLong(self):
        self.wide.update("test")
        w = config_value('BASE', 'LONG')
        self.assertEqual(len(w), 4)
        self.assertEqual(w, 'test')
    
    def testDelete(self):
        remember = self.wide.setting.id
        self.wide.update('woot')
        
        try:
            q = LongSetting.objects.get(pk = remember)
            self.fail("Should be deletec")
        except LongSetting.DoesNotExist:
            pass
            
class OverrideTest(TestCase):
    """Test settings overrides"""
    def setUp(self):
        # clear out cache from previous runs
        keyedcache.cache_delete()
    
        djangosettings.LIVESETTINGS_OPTIONS = {
            1 : {
                'DB' : False,
                'SETTINGS' : {
                        'overgroup' : {
                            's2' : '100',
                            'choices' : '["one","two","three"]'
                        }
                    }
                }
            }
    
        g = ConfigurationGroup('overgroup','Override Group')
        self.g = g
        config_register(StringValue(g, 's1'))
        config_register(IntegerValue(g, 's2', default=10))
        config_register(IntegerValue(g, 's3', default=10))
        config_register(MultipleStringValue(g, 'choices'))
        
    def tearDown(self):
        djangosettings.LIVESETTINGS_OPTIONS = {}

    def testOverriddenSetting(self):
        """Accessing an overridden setting should give the override value."""
        c = config_get('overgroup', 's2')
        self.assertEquals(c.value, 100)    

    def testCantChangeSetting(self):
        """When overridden, setting a value should not work, should get the overridden value"""
        c = config_get('overgroup', 's2')
        c.update(1)

        c = config_get('overgroup', 's2')
        self.assertEquals(c.value, 100)
        
    def testNotOverriddenSetting(self):
        """Settings which are not overridden should return their defaults"""
        c = config_get('overgroup', 's3')
        
        self.assertEquals(c.value, 10)
        
    def testOverriddenListSetting(self):
        """Make sure lists work when overridden"""
        
        c = config_get('overgroup', 'choices')
        v = c.value
        self.assertEqual(len(v), 3)
        self.assertEqual(v[0], "one")
        self.assertEqual(v[1], "two")
        self.assertEqual(v[2], "three")



from shipping.modules.tiered.models import Carrier, ShippingTier, CarrierTranslation
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _

class ShippingTier_Inline(admin.TabularInline):
    model = ShippingTier
    extra = 6
    ordering = ('min_total', 'expires')

class CarrierTranslation_Inline(admin.TabularInline):
    model = CarrierTranslation
    ordering = ('carrier',)
    extra = 1

class CarrierOptions(admin.ModelAdmin):
    ordering = ('key',)
    inlines = [CarrierTranslation_Inline, ShippingTier_Inline, ]

admin.site.register(Carrier, CarrierOptions)





# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import optparse
import os
import sys

from gazpacho import __version__
from gazpacho.i18n import _

PYGTK_REQUIRED = (2, 6, 0)
KIWI_REQUIRED = (1, 9, 6)

def open_project(app, filename, profile):
    if not profile:
        return app.open_project(filename)

    print('profiling')
    import hotshot
    from hotshot import stats
    prof = hotshot.Profile("gazpacho.prof")
    prof.runcall(app.open_project, filename)
    prof.close()

    s = stats.load("gazpacho.prof")
    s.strip_dirs()
    s.sort_stats('time', 'calls')
    s.print_stats(25)

def run_batch(app, command):
    ns = dict(new=lambda w: app.create(w))
    eval(command, {}, ns)

def run_console(app):
    try:
        import readline
        import rlcompleter
        assert rlcompleter
        readline.parse_and_bind('tab: complete')
    except ImportError:
        pass

    import code
    ia = code.InteractiveConsole(
        locals=dict(project=app.get_current_project()))
    ia.interact()

def check_deps(debug=False):
    # PyGTK
    if debug:
        print('Importing PyGTK...')

    try:
        import gtk
        gtk # stuid pyflakes
    except ImportError:
        try:
            import pygtk
            # This modifies sys.path
            pygtk.require('2.0')
            # Try again now when pygtk is imported
            import gtk
        except ImportError:
            raise SystemExit("PyGTK is required to run Gazpacho")

    import gobject
    if (os.path.dirname(os.path.dirname(gtk.__file__)) !=
        os.path.dirname(gobject.__file__)):
        print ('WARNING: GTK+ and GObject modules are not loaded from '
               'the same prefix')

    if debug:
        print('Python:\t', '.'.join(map(str, sys.version_info[:3])))
        print('GTK+:\t', '.'.join(map(str, gtk.gtk_version)))
        print('PyGTK:\t', '.'.join(map(str, gtk.pygtk_version)))

    if gtk.pygtk_version < PYGTK_REQUIRED:
        raise SystemExit("PyGTK 2.6.0 or higher required to run Gazpacho")

    if gtk.pygtk_version >= (2, 7, 0):
        if gtk.pygtk_version < (2, 8, 0):
            raise SystemExit("PyGTK 2.7.x is not supported, upgrade to 2.8.0")

        if debug:
            print('Using PyGTK 2.8.x, ignoring deprecation warnings')

        # Ignore deprecation warnings when using 2.7.x
        import warnings
        warnings.filterwarnings('ignore', category=DeprecationWarning)

    # kiwi
    if debug:
        print('Importing kiwi...')

    try:
        from kiwi.__version__ import version
    except ImportError:
        raise SystemExit("kiwi is required to run Gazpacho")

    if version < KIWI_REQUIRED:
        raise SystemExit("kiwi %s or higher is required" %
                         '.'.join(map(str, KIWI_REQUIRED)))

def setup_app(testsuite=False):
    pass

def show(options, filenames=[]):
    import gtk
    from gazpacho.catalog import load_catalogs
    load_catalogs()
    from gazpacho.loader.loader import ObjectBuilder

    if not filenames:
        raise SystemExit('--show needs at least one filename')

    windows = []
    def window_close_cb(window, event):
        windows.remove(window)
        if not windows:
            gtk.main_quit()

    for filename in filenames:
        builder = ObjectBuilder(filename)
        for widget in builder.get_widgets():
            if not isinstance(widget, gtk.Window):
                continue
            elif isinstance(widget, gtk.Action):
                if widget.get_property('stock-id') == gtk.STOCK_QUIT:
                    widget.connect('activate', gtk.main_quit)
            widget.connect('delete-event', window_close_cb)
            widget.show_all()
            windows.append(widget)

    if not windows:
        raise SystemExit("none of the specified files has any windows")

    gtk.main()

def debug_hook(exctype, value, tb):
    import traceback

    traceback.print_exception(exctype, value, tb)
    print()
    print('-- Starting debugger --')
    print()
    import pdb
    pdb.pm()
    raise SystemExit

def launch(options, filenames=[]):
    if options.debug:
        print('Loading gazpacho')

    # Delay imports, so command line parsing is not slowed down
    from kiwi.component import provide_utility
    from gazpacho.interfaces import IGazpachoApp, IPluginManager
    from gazpacho.app.app import Application
    from gazpacho.app.debugwindow import DebugWindow, show
    from gazpacho.plugins import PluginManager

    plugin_manager = PluginManager()
    provide_utility(IPluginManager, plugin_manager)

    gazpacho = Application()
    provide_utility(IGazpachoApp, gazpacho)

    if options.debug:
        sys.excepthook = debug_hook
    else:
        DebugWindow.application = gazpacho
        sys.excepthook = show

    for filename in filenames:
        if not os.path.exists(filename):
            raise SystemExit('%s: no such a file or directory' % filename)

        if not os.access(filename, os.R_OK):
            raise SystemExit('Could not open file %s: Permission denied.' %
                             filename)
        open_project(gazpacho, filename, options.profile)

    if options.update:
        for project in gazpacho.get_projects():
            project.save(project.path)

        return

    # If no filenames were specified, open up an empty project
    if not filenames:
        gazpacho.new_project()

    if options.batch:
        run_batch(gazpacho, options.batch)
        return
    elif options.console:
        run_console(gazpacho)
        return

    if options.debug:
        print('Running gazpacho')

    gazpacho.run()

def main(args=[]):
    parser = optparse.OptionParser(version=__version__)
    parser.add_option('', '--profile',
                      action="store_true",
                      dest="profile",
                      help=_("Turn on profiling support"))
    parser.add_option('', '--debug',
                      action="store_true",
                      dest="debug",
                      help=_("Turn on pdb debugging support"))
    parser.add_option('', '--batch',
                      action="store",
                      dest="batch",
                      help=_("Batch command"))
    parser.add_option('-u', '--update',
                      action="store_true",
                      dest="update",
                      help=_("Load glade file and save it"))
    parser.add_option('-c', '--console',
                      action="store_true",
                      dest="console",
                      help=_("Start up a console"))
    parser.add_option('-s', '--show',
                      action="store_true",
                      dest="show",
                      help=_("Only open and display the interface"))

    # Until we can use GOptionParser
    args = args[:]
    if '--g-fatal-warnings' in args:
        args.remove('--g-fatal-warnings')

    options, args = parser.parse_args(args)

    if options.batch or options.console:
        options.debug = True

    filenames = []
    if len(args) >= 2:
        filenames = [os.path.abspath(name) for name in args[1:]]

    # Do this before importing application, which imports gtk
    check_deps(debug=options.debug)
    setup_app()

    if options.show:
        show(options, filenames)
    else:
        launch(options, filenames)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2019 Etienne Robillard <tkadm30@yandex.ru>
# All rights reserved.
# For experimental use only... Require CPython > 3.4
"""AsyncIOController API Version 0.8.2

"""

import sys
import logging
import asyncio
from asyncio import get_event_loop
from notmm.controllers.wsgi import WSGIController
from notmm.controllers.base import request, sessionmanager

log = logging.getLogger('asyncio')
log.setLevel(logging.DEBUG)


class AsyncIOController(WSGIController):
    def __init__(self, settings=None, executor=None, loop=None
        ):
        super(AsyncIOController, self).__init__(settings)

    def get_response(self, request=None, method='GET', data={}):
        return super(AsyncIOController, self).get_response(request)
    
    @asyncio.coroutine
    def application(self, environ, start_response):
        with sessionmanager(environ):
            request.environ.update(environ)
            response = self.get_response(request=request)
        return response(environ, start_response)
    
    @asyncio.coroutine
    def __call__(self, environ, start_response):
        return self.application(environ, start_response)



from django.contrib.sites.models import Site
from django.core import urlresolvers
from django.db import models
from django.db.models.fields.files import FileField
from django.utils.encoding import smart_str
from django.utils.hashcompat import sha_constructor
from django.utils.translation import ugettext_lazy as _
from livesettings import config_value
from product import signals
from product.models import Product
import product.modules.downloadable.config
from satchmo_store.shop.models import Order
from satchmo_utils import normalize_dir
import datetime
import os
import random

SATCHMO_PRODUCT=True

def get_product_types():
    return ('DownloadableProduct',)


def _protected_dir(instance, filename):
    # if we get a SettingNotSet exception (even though we've already
    # imported/loaded it), that's bad, so let it bubble up.
    raw = config_value('PRODUCT', 'PROTECTED_DIR')
    updir = os.path.normpath(normalize_dir(raw))
    return os.path.join(updir, instance.file.field.get_filename(filename))

class DownloadableProduct(models.Model):
    """
    This type of Product is a file to be downloaded
    """
    product = models.OneToOneField(Product, verbose_name=_("Product"), primary_key=True)
    file = FileField(_("File"), upload_to=_protected_dir)
    num_allowed_downloads = models.IntegerField(
        _("Num allowed downloads"),
        help_text=_("Number of times link can be accessed. Enter 0 for unlimited."),
        default=0)
    expire_minutes = models.IntegerField(
        _("Expire minutes"),
        help_text=_("Number of minutes the link should remain active. Enter 0 for unlimited."),
        default=0)
    active = models.BooleanField(_("Active"), help_text=_("Is this download currently active?"), default=True)
    is_shippable = False
    is_downloadable = True

    def __unicode__(self):
        return self.product.slug

    def _get_subtype(self):
        return 'DownloadableProduct'

    def create_key(self):
        salt = sha_constructor(str(random.random())).hexdigest()[:5]
        download_key = sha_constructor(salt+smart_str(self.product.name)).hexdigest()
        return download_key

    def order_success(self, order, order_item):
        signals.subtype_order_success.send(self, product=self, order=order, subtype="download")

    class Meta:
        verbose_name = _("Downloadable Product")
        verbose_name_plural = _("Downloadable Products")

    def save(self, *args, **kwargs):
        if hasattr(self.product,'_sub_types'):
            del self.product._sub_types
        super(DownloadableProduct, self).save(*args, **kwargs)

class DownloadLink(models.Model):
    downloadable_product = models.ForeignKey(DownloadableProduct, verbose_name=_('Downloadable product'))
    order = models.ForeignKey(Order, verbose_name=_('Order'))
    key = models.CharField(_('Key'), max_length=40)
    num_attempts = models.IntegerField(_('Number of attempts'), )
    time_stamp = models.DateTimeField(_('Time stamp'), )
    active = models.BooleanField(_('Active'), default=True)

    def _attempts_left(self):
        return self.downloadable_product.num_allowed_downloads - self.num_attempts
    attempts_left = property(_attempts_left)

    def is_valid(self):
        # Check num attempts and expire_minutes
        if not self.downloadable_product.active:
            return (False, _("This download is no longer active"))
        maxattempts = self.downloadable_product.num_allowed_downloads
        if maxattempts > 0 and self.num_attempts >= maxattempts:
            return (False, _("You have exceeded the number of allowed downloads."))
        expiremins = self.downloadable_product.expire_minutes
        expire_time = datetime.timedelta(minutes=expiremins) + self.time_stamp
        if expiremins > 0 and datetime.datetime.now() > expire_time:
            return (False, _("This download link has expired."))
        return (True, "")

    def get_absolute_url(self):
        return('satchmo_store.shop.views.download.process', (), { 'download_key': self.key})
    get_absolute_url = models.permalink(get_absolute_url)

    def get_full_url(self):
        url = urlresolvers.reverse('satchmo_download_process', kwargs= {'download_key': self.key})
        return('http://%s%s' % (Site.objects.get_current(), url))

    def save(self, **kwargs):
        """
       Set the initial time stamp
        """
        if self.time_stamp is None:
            self.time_stamp = datetime.datetime.now()
        super(DownloadLink, self).save(**kwargs)

    def __unicode__(self):
        return u"%s - %s" % (self.downloadable_product.product.slug, self.time_stamp)

    def _product_name(self):
        return u"%s" % (self.downloadable_product.product.translated_name())
    product_name=property(_product_name)

    class Meta:
        verbose_name = _("Download Link")
        verbose_name_plural = _("Download Links")


import config
import listeners
listeners.start_default_listening()



# -*- coding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from livesettings import ConfigurationGroup, StringValue, config_register

PRODUCT_GROUP = ConfigurationGroup('PRODUCT', _('Product Settings'))

config_register(
    StringValue(PRODUCT_GROUP,
        'PROTECTED_DIR',
        description=_("Protected dir"),
        help_text=_("""This is only used if you use Downloadable Products.
This value will be appended to MEDIA_ROOT/MEDIA_URL.  Do not worry about slashes.
We can handle it any which way."""),
        default="protected",
    ))





from notmm.utils.django_settings import SettingsProxy
from test_support import unittest
from django.conf import settings as _settings

class LazySettingsTestCase(unittest.TestCase):
    
    def setUp(self):
        self.settings_module = _settings

    
    def test_count(self):
        # check that django.conf.settings and site-derived
        # settings instance are equivalent
        from sandbox.configuration import settings
        
        c1 = settings.count()
        c2 = len([item for item in dir(self.settings_module) if item.isupper() ])
        
        # django will add "SETTINGS_MODULE" dynamically
        #self.failUnlessEqual(c1, c2 - 1)

    def test_equivalent(self):

        settings = SettingsProxy(autoload=True).settings
        self.failUnlessEqual(hasattr(self.settings_module, 'SETTINGS_MODULE'), True)
        self.failUnlessEqual(hasattr(settings, 'SETTINGS_MODULE'), False)
        
        # check that LazySettings() works as expected; SETTINGS_MODULE value
        # must match DJANGO_SETTINGS_MODULE..
        self.failUnlessEqual(self.settings_module.SETTINGS_MODULE, settings.__name__)




from decimal import Decimal
from django.contrib.sites.models import Site
from django.db import models
from django.test import TestCase
from livesettings import config_value
from product.models import *
from satchmo_store.shop.models import *
from shipping.modules.flat.shipper import Shipper as flat
from shipping.modules.per.shipper import Shipper as per
import keyedcache

class ShippingBaseTest(TestCase):

    fixtures = ['l10n-data.yaml','test_shop.yaml']

    def setUp(self):
        self.site = Site.objects.get_current()
        self.product1 = Product.objects.create(slug='p1', name='p1', site=self.site)
        self.cart1 = Cart.objects.create(site=self.site)
        self.cartitem1 = self.cart1.add_item(self.product1, 3)
        
    def tearDown(self):
        keyedcache.cache_delete()
        
    def test_downloadable_zero_shipping(self):
        subtypes = config_value('PRODUCT','PRODUCT_TYPES')
        if "product::DownloadableProduct" in subtypes:
            subtype2 = DownloadableProduct.objects.create(product=product1)
            self.assertEqual(product1.get_subtypes(), ('ConfigurableProduct', 'DownloadableProduct'))
            
            self.assertFalse(subtype2.is_shippable)
            self.assertFalse(self.product1.is_shippable)
            self.assertFalse(self.cart1.is_shippable)
            self.assertEqual(flat(self.cart1, None).cost(), Decimal("0.00"))
            self.assertEqual(per(self.cart1, None).cost(), Decimal("0.00"))


    def test_simple_shipping(self):
        # Product.is_shippable should be True unless the Product has a subtype
        # where is_shippable == False
        subtype1 = ConfigurableProduct.objects.create(product=self.product1)
        self.assert_(getattr(subtype1, 'is_shippable', True))
        self.assert_(self.cartitem1.is_shippable)
        self.assert_(self.product1.is_shippable)
        self.assert_(self.cart1.is_shippable)
        self.assertEqual(flat(self.cart1, None).cost(), Decimal("4.00"))
        self.assertEqual(per(self.cart1, None).cost(), Decimal("12.00"))



"""Base urls used by Satchmo.

Split out from urls.py to allow much easier overriding and integration with larger apps.
"""
from django.conf.urls.defaults import *
#from signals_ahoy.signals import collect_urls
#from product.urls.base import adminpatterns as prodpatterns
#from shipping.urls import adminpatterns as shippatterns
import logging
import satchmo_store

log = logging.getLogger('shop.urls')

urlpatterns = patterns('',
    (r'^admin/doc/', include('django.contrib.admindocs.urls')),
    (r'^accounts/', include('satchmo_store.accounts.urls')),
    (r'^settings/', include('livesettings.urls')),
    (r'^cache/', include('keyedcache.urls')),
)

#collect_urls.send(sender=satchmo_store, patterns=urlpatterns)



from threading import local
from functools import wraps
 
# support utilities
from test_support import (\
    unittest, 
    #setup_test_handlers,
    BaseControllerTestCase, 
    TestClient,
    make_app
    )

from notmm.controllers.wsgi import WSGIController
from notmm.utils.wsgilib import HTTPResponse

from controller import DAV10Controller

class DAV10Client(TestClient):
    pass

class DAV10ControllerTestCase(BaseControllerTestCase):

    wsgi_app = DAV10Controller # The wrapped HTTP/DAV wsgi handler
    
    def __init__(self, methodName='runTest'):
        unittest.TestCase.__init__(self, methodName)
    
    def test_make_app(self):

        self.callback = make_app(self.wsgi_app)
        self.client = DAV10Client(self.callback)
 
        




from notmm.utils.urlmap import RegexURLMap, url

urlpatterns = RegexURLMap()
urlpatterns.add_routes('', 
        #(r'^login/$', 'moviereviews.views.login', {
        #    'template_name' : 'moviereviews/login.mako'
        #}),

        url(r'^test_post$', \
            'moviereviews.views.test_schevo_database', {
            'template_name' : 'moviereviews/test.mako'
        }, name="test_post"),
)
urlpatterns.commit()




# Copyright (C) 2006 Mattias Karlsson
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

class SignalHandlerStorage:
    """
    The signal storage is used to keep track of signal handlers and
    provides a way to disconnect all of them in an simple way.
    """

    def __init__(self):
        """
        Initialize the signal storage.
        """
        self._handlers = {}

    def connect(self, obj, signal, callback, *args):
        """
        Connect a signal handler. All handlers connected in this way
        can be disconnected by calling disconnet().

        @param obj: the object to connect the handler to
        @type obj: gobject.GObject
        @param signal: the signal to listen for
        @type signal: str
        @param callback: the callback method
        @type callback: function object
        """
        handler_id = obj.connect(signal, callback, *args)
        self._handlers.setdefault(obj, []).append(handler_id)

    def disconnect_all(self):
        """
        Disconnect all handlers.
        """
        for obj, handler_ids in self._handlers.items():
            for hid in handler_ids:
                obj.disconnect(hid)

        self._handlers = {}



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""This module implements a Clipboard object to add support for
cut/copy/paste operations in Gazpacho.
It also defines a ClipboardWindow widget to graphically display and
select Clipboard data.

The Clipboard holds instances of ClipboardItems which contains information
of the widgets and a XML representation of them.
"""

import gobject
import gtk
from kiwi.utils import gsignal, type_register

from gazpacho import util

from gazpacho.command import FlipFlopCommandMixin, Command
from gazpacho.commandmanager import command_manager
from gazpacho.i18n import _

class ClipboardItem(object):
    """The ClipboardItem is the data that is stored in the
    clipboard. It contains an XML representation of the widget and
    some additional information that can be useful to know about
    without having to recreate the widget.."""

    def __init__(self, gadget):
        self.name = gadget.name
        self.icon = gadget.adaptor.icon.get_pixbuf()
        self.is_toplevel = gadget.is_toplevel()
        self.type = gadget.adaptor.type

        self._xml = gadget.to_xml(skip_external_references=True)

    def to_xml(self):
        return self._xml

class Clipboard(gobject.GObject):
    """The Clipboard is an abstraction Gazpacho uses to support
    cut/copy/paste operations. It's basically a container which
    holds XML representation of copied/cut widgets.

    So far it doesn't use X selection mechanism but it should
    not be dificult to add it in the future.
    """

    gsignal('widget-added', object)
    gsignal('widget-removed', object)
    gsignal('selection-changed', object)

    def __init__(self):
        gobject.GObject.__init__(self)

        # A list with the clipboard content. New items are added to
        # the end of the list.
        self.content = []

        # The currently selected item. If None we use the last item in
        # the content list instead.
        self._selected = None

        # We limit the maximum number of items in the clipboard
        # otherwise it would grow indefinitely. If to many items are
        # added the oldest will be removed.
        self._max_clipboard_items = 15

    def get_selected(self):
        return self._selected

    def set_selected(self, item):
        self._selected = item
        if not item and self.content:
            item = self.content[-1]
        self.emit('selection_changed', item)

    selected = property(get_selected, set_selected)

    def add_gadget(self, src_gadget):
        """Add a ClipboardItem wrapping the the source widget to the clipboard.
        A 'widget-added' signal is then emitted to indicate that there
        are new data on the clipboard. If the clipboard contains too
        many items the oldest item will be removed and a
        'widget-removed' signal emitted."""
        item = ClipboardItem(src_gadget)

        self.content.append(item)
        self.emit('widget-added', item)
        self.selected = item

        # Remove the oldest item if the clipboard contains too many
        # items
        if len(self.content) > self._max_clipboard_items:
            self._remove_oldest_item()

    def _remove_oldest_item(self):
        """Remove the oldest item from the clipboard."""
        if not self.content:
            return

        item_removed = self.content.pop(0)
        self.emit('widget-removed', item_removed)

    def get_selected_item(self):
        """Get the selected clipboard item."""
        if self.selected:
            return self._selected
        elif self.content:
            return self.content[-1]
        return None

    def get_selected_widget(self, project):
        """Get a widget instance from the selected item.

        This build a widget instance from the XML of the selected item.
        Before adding this to a project you should check that its name
        is unique
        """


        item = self.get_selected_item()
        if item is None:
            return

        from gazpacho.gadget import Gadget
        return Gadget.from_xml(project, item.to_xml(), item.name)

    # public methods for copy/cut/paste

    def copy(self, gadget):
        """Add a copy of the widget to the clipboard.

        Note that it does not make sense to undo this operation
        """
        # internal children cannot be copied. Shoulw we notify the user?
        if gadget.internal_name is not None:
            return

        self.add_gadget(gadget)

    def cut(self, gadget):
        # internal children cannot be cut. Should we notify the user?
        if gadget.internal_name is not None:
            return

        # add it to the clipboard
        self.add_gadget(gadget)

        project = gadget.project
        if isinstance(gadget.widget, gtk.TreeViewColumn):
            from gazpacho.util import get_parent
            from gazpacho.widgets.base.treeview import CommandAddRemoveTreeViewColumn
            tree_view = get_parent(gadget.widget)
            cmd = CommandAddRemoveTreeViewColumn(tree_view, gadget,
                                                 project, False)
        else:
            cmd = CommandCutPaste(gadget, project, None, True)

        command_manager.execute(cmd, project)

    def paste(self, placeholder, project):
        if project is None:
            raise ValueError("No project has been specified. Cannot paste "
                             "the widget")

        gadget = self.get_selected_widget(project)

        if (isinstance(placeholder, gtk.TreeView)
            and isinstance(gadget.widget, gtk.TreeViewColumn)):
            from gazpacho.gadget import Gadget
            from gazpacho.widgets.base.treeview import CommandAddRemoveTreeViewColumn
            tree_view = Gadget.from_widget(placeholder)
            cmd = CommandAddRemoveTreeViewColumn(tree_view, gadget,
                                                 project, True)
        else:
            cmd = CommandCutPaste(gadget, project, placeholder, False)

        command_manager.execute(cmd, project)
        return gadget

type_register(Clipboard)

clipboard = Clipboard()

class ClipboardWindow(gtk.Window):
    """ClipboardWindow is a Widget to represent and manage a Clipboard.

    It displays the current Clipboard contents through a TreeView.
    """

    ICON_COLUMN = 0
    NAME_COLUMN = 1
    WIDGET_COLUMN = 2

    def __init__(self, parent, clipboard):
        gtk.Window.__init__(self)
        self.set_title(_('Clipboard Contents'))
        self.set_transient_for(parent)

        self._clipboard = clipboard
        clipboard.connect('widget-added', self._widget_added_cb)
        clipboard.connect('widget-removed', self._widget_removed_cb)

        self._store = self._create_store(clipboard)
        self._store.connect('row-changed', self._row_changed_cb)

        self._view = self._create_view()
        self._view.get_selection().connect('changed',
                                           self._selection_changed_cb)
        self._view.connect('row-activated', self._row_activated_cb)
        self._set_default_selection()

        scroll = gtk.ScrolledWindow()
        scroll.set_size_request(300, 200)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)
        scroll.add(self._view)
        self.add(scroll)

        self._view.show_all()

    def _create_store(self, clipboard):
        """Create a ListStore and fill it will items from the clipboard."""
        store = gtk.ListStore(gtk.gdk.Pixbuf, str, object)
        for clip_item in clipboard.content:
            store.append((clip_item.icon, clip_item.name, clip_item))
        return store

    def _create_view(self):
        """Create the tree view that will display the clipboard data."""
        view = gtk.TreeView(self._store)
        view.set_headers_visible(False)

        column = gtk.TreeViewColumn()
        renderer1 = gtk.CellRendererPixbuf()
        column.pack_start(renderer1, False)
        column.add_attribute(renderer1, 'pixbuf', ClipboardWindow.ICON_COLUMN)
        renderer2 = gtk.CellRendererText()
        column.pack_start(renderer2)
        column.add_attribute(renderer2, 'text', ClipboardWindow.NAME_COLUMN)

        view.append_column(column)
        return view

    def _set_clipboard_selection(self):
        """Notify the Clipboard about which item is currently selected."""
        (model, it) = self._view.get_selection().get_selected()
        if it:
            clip_item = model[it][ClipboardWindow.WIDGET_COLUMN]
            self._clipboard.selected = clip_item

    def _selection_changed_cb(self, selection, data=None):
        """Callback for selection changes in the tree view."""
        self._set_clipboard_selection()

    def _widget_added_cb(self, clipboard, new_item):
        """Callback that is triggered when an item is added to the
        clipboard."""
        self._store.append((new_item.icon, new_item.name, new_item))

    def _widget_removed_cb(self, clipboard, old_item):
        """Callback that is triggered when an item is removed from the
        clipboard."""
        # It's always the oldes item that is removed so we can just
        # remove the first item from the store.
        first_iter = self._store.get_iter_first()
        if first_iter:
            self._store.remove(first_iter)

    def _row_changed_cb(self, model, path, new_iter, data=None):
        """Callback that is called when rows are added or removed."""
        self._set_default_selection()

    def _set_default_selection(self):
        """Set the default selection, i.e. the latest additiion."""
        last_iter = get_last_iter(self._store)
        if last_iter:
            self._view.get_selection().select_iter(last_iter)

    def _row_activated_cb(self, treeview, path, column):
        item = self._store[path][2]
        print('The current item is represented by:')
        print(item.to_xml())

    def hide_window(self):
        """Hide the clipboard window."""
        self.hide()
        self._clipboard.selected = None

    def show_window(self):
        """Show the clipboard window."""
        self.show_all()
        self._set_default_selection()

def get_last_iter(model):
    """Convenient function to get the last iterator from a tree model."""
    items = len(model)
    if items > 0:
        return model.iter_nth_child(None, items - 1)
    return None


class CommandCutPaste(FlipFlopCommandMixin, Command):

    def __init__(self, gadget, project, placeholder, cut):
        FlipFlopCommandMixin.__init__(self, cut)

        if cut:
            description = _('Cut widget %s into the clipboard') % gadget.name
        else:
            description = _('Paste widget %s from the clipboard') % gadget.name
        Command.__init__(self, description)

        self._project = project
        self._placeholder = placeholder
        self._gadget = gadget

    def _execute_cut(self):
        from gazpacho.gadget import Gadget
        from gazpacho.placeholder import Placeholder

        gadget = self._gadget

        if not gadget.is_toplevel():
            parent = gadget.get_parent()

            if not self._placeholder:
                self._placeholder = Placeholder()

            Gadget.replace(gadget.widget,
                           self._placeholder, parent)

        gadget.widget.hide()
        gadget.project.remove_widget(gadget.widget)
        gadget.deleted = True

    _execute_state1 = _execute_cut

    def _execute_paste(self):
        # Note that updating the dependencies might replace the
        # widget's gtk-widget so we need to make sure we refer to the
        # correct one afterward.
        from gazpacho.gadget import Gadget
        self._gadget.deleted = False

        if self._gadget.is_toplevel():
            project = self._project
        else:
            parent = util.get_parent(self._placeholder)
            project = parent.project
            Gadget.replace(self._placeholder,
                           self._gadget.widget,
                           parent)

        project.add_widget(self._gadget.widget, new_name=True)
        self._gadget.select()

        self._gadget.widget.show_all()

        # We need to store the project of a toplevel widget to use
        # when undoing the cut.
        self._project = project

        return self._gadget

    _execute_state2 = _execute_paste



from decimal import Decimal
from shipping.config import shipping_method_by_key

def update_shipping(order, shipping, contact, cart):
    """Set the shipping for this order"""
    # Set a default for when no shipping module is used
    order.shipping_cost = Decimal("0.00")

    # Save the shipping info
    shipper = shipping_method_by_key(shipping)
    shipper.calculate(cart, contact)
    order.shipping_description = shipper.description().encode("utf-8")
    order.shipping_method = shipper.method()
    order.shipping_cost = shipper.cost()
    order.shipping_model = shipping



# Django Signals Ahoy
# Copyright Bruce Kroeze 2009
#
# Released under the "New BSD License"
# Please see LICENSE.rst in this distribution for license details.
# -----------------

VERSION = (0, 1, 1)

# Dynamically calculate the version based on VERSION tuple
if len(VERSION)>2 and VERSION[2] is not None:
    str_version = "%d.%d_%s" % VERSION[:3]
else:
    str_version = "%d.%d" % VERSION[:2]

__version__ = str_version


### Base request handler to implement a WSGI gateway 
from BaseHTTPServer import BaseHTTPRequestHandler

#### PyWebDAV imports
#import DAV
#import DAV.WebDAVServer
#DAV.WebDAVServer.DEBUG = True
####
# DAV server extention to implement DAV responses
# for published realms
from pywebdav.server.fileauth import DAVAuthHandler
####

####
# Front end controller script to integrate the DAV fileserver
# API 
from notmm.controllers.wsgi import WSGIController
from notmm.utils.wsgilib import HTTPRequest, HTTPResponse
####

#Local modules
#import settings
#import utils

# DAV10_CONFIG = getattr(settings, 'DAV10_CONFIG', {})
# Setup the filesystem-based provider :-)
# DAV10_IFACE_CLASS = settings.MOUNTPOINTS['test']['root']

class DAV10RequestHandler(BaseHTTPRequestHandler, DAVAuthHandler):
    
    server_software = "Python DAVserver/1.0 (WSGI version)"

    def __init__(self, *args, **kwargs):
        
        #self._config = None

        DAVAuthHandler.__init__(self, *args, **kwargs)


    def handle(self):
        DAVAuthHandler.handle(self)
    
class DAV10Controller(WSGIController):
    request_class = HTTPRequest
    response_class = HTTPResponse

    def application(self, environ, start_response):
        super(DAV10Controller, self).application(environ, start_response)





"""
Canadian postal code validation. The structure of the codes is at:

http://www.canadapost.ca/tools/pg/manual/pgaddress-e.asp#1380891

"""
import re

from django.utils.translation import ugettext_lazy as _

CA_POSTCODE_RE = re.compile(r'^(?P<FSA>[ABCEGHJKLMNPRSTVXYZ]\d[A-Z])\s*(?P<LDU>\d[A-Z]\d)$')
def validate(postcode):
    postcode = postcode.upper()
    m = CA_POSTCODE_RE.match(postcode)
    if m:
        return m.group('FSA') + m.group('LDU')
    else:
        raise ValueError(_('Invalid postal code for Canada'))



# Copyright (C) 2005 by SICEm S.L. and Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk

from kiwi.ui.dialogs import BaseDialog

from gazpacho.command import FlipFlopCommandMixin, Command
from gazpacho.commandmanager import command_manager
from gazpacho.i18n import _
from gazpacho.model import ModelEditorDialog, GazpachoModel, renderer_expert
from gazpacho.properties import prop_registry, CustomProperty, StringType, \
     TransparentProperty
from gazpacho.propertyeditor import PropertyCustomEditorWithDialog, \
     PropertyEditorDialog
from gazpacho.gadget import Gadget, load_gadget_from_widget
from gazpacho.widgetadaptor import WidgetAdaptor, CreationCancelled
from gazpacho.widgetregistry import widget_registry
from gazpacho.widgets.base.base import ContainerAdaptor

class TreeViewAdaptor(ContainerAdaptor):
    # note that the treeview does not have a real model until a column is
    # added to it
    def create(self, context, interactive=True):
        # add a model to the project
        project = context.get_project()
        model = project.model_manager.create_model()

        # this (empty) model is needed to show the '+' column
        tree_view = gtk.TreeView(model.get_model())

        self._add_adder_column(tree_view)

        return tree_view

    def fill_empty(self, context, widget):
        pass

    def _on_add(self, column, gtk_tree_view):
        gadget = Gadget.from_widget(gtk_tree_view)
        self.add_tree_view_column(gadget, True)

    def add_tree_view_column(self, tree_view, interactive=True):
        adaptor = widget_registry.get_by_type(gtk.TreeViewColumn)
        project = tree_view.project
        try:
            column = Gadget(adaptor, project)
            column.create_widget(interactive)
        except CreationCancelled:
            return

        cmd = CommandAddRemoveTreeViewColumn(tree_view, column, project, True)
        command_manager.execute(cmd, project)
        return column

    def _add_adder_column(self, widget):
        col = gtk.TreeViewColumn()
        image = gtk.Image()
        image.show()
        image.set_from_stock(gtk.STOCK_ADD, gtk.ICON_SIZE_BUTTON)
        col.set_widget(image)
        col.set_clickable(True)
        col.set_reorderable(False)
        col.connect('clicked', self._on_add, widget)
        widget.append_column(col)

    def get_children(self, context, tree_view):
        return [c for c in tree_view.get_columns() \
                if isinstance(c, GazpachoTreeViewColumn)]

    def load(self, context, tree_view):
        project = context.get_project()
        model = tree_view.get_model()
        gzmodel = None
        if model:
            gzmodel = project.model_manager.get_model(model.name)

        gadget = Gadget.load(tree_view, project)

        if gzmodel:
            gadget.widget.set_model(gzmodel.get_model())

        for column in tree_view.get_columns():
            load_gadget_from_widget(column, project)

        self._add_adder_column(tree_view)

        return gadget

    def replace_child(self, context, current, new, container):
        # don't put placeholders in the columns
        container.remove_column(current)

    def add_child(self, tree_view, column):
        cols = len(tree_view.widget.get_columns())
        column.widget.set_widget(None)
        tree_view.widget.insert_column(column.widget, cols - 1)
        column.widget.update_widget()

        # update the model with this column
        self.update_model_with_column(tree_view, column)

    def remove_child(self, tree_view, column):
        index = self.get_column_index(tree_view, column)
        tree_view.widget.remove_column(column.widget)

        self.update_model_with_column(tree_view, column, False)

        offset = len(column.widget.get_cell_renderers())
        self.update_other_columns_attributes(tree_view, -offset, index)

    def update_model_with_column(self, tree_view, column, adding=True):
        model = self.get_model(tree_view)

        if adding:
            model.add_columns_for_tree_view_column(column.widget)
        else:
            model.remove_columns_of_tree_view_column(column.widget)

        tree_view.widget.set_model(model.get_model())

    def before_change_column_layout(self, tree_view, column):
        """Remove the column renderers from the model"""
        index = self.get_column_index(tree_view, column)
        self.update_model_with_column(tree_view, column, False)

        offset = len(column.widget.get_cell_renderers())
        self.update_other_columns_attributes(tree_view, -offset, index)

    def after_change_column_layout(self, tree_view, column):
        """Inset the column renderers to the model at the offset
        this column is"""
        index = self.get_column_index(tree_view, column)
        cells = 0
        for i in range(index):
            col = tree_view.widget.get_column(i)
            cells += len(col.get_cell_renderers())

        # we update the model by *inserting* the types (not appending)
        model = self.get_model(tree_view)
        model.insert_columns_of_tree_view_column(column.widget, cells)
        tree_view.widget.set_model(model.get_model())

        # not we shift the attributes value of the renderers that are after the
        # ones of the column
        offset = len(column.widget.get_cell_renderers())
        self.update_other_columns_attributes(tree_view, offset, index + 1)

    def update_other_columns_attributes(self, tree_view, offset, column_index):
        """When removing a column from the model all other columns that are
        after it will have their attributes wrong (e.g. text=3 will be text=2)
        """
        # the last column is the adder column and we don't use it
        columns = tree_view.widget.get_columns()[column_index:-1]
        for c in columns:
            c.update_attributes(offset)

    def get_column_index(self, tree_view, column):
        """Return the index of column inside tree_view"""
        for i, col in enumerate(tree_view.widget.get_columns()):
            if col is column.widget:
                return i

    def get_model(self, tree_view):
        return GazpachoModel.from_model(tree_view.widget.get_model())

    def delete(self, context, widget):        
        model = GazpachoModel.from_model(widget.get_model())
        context.get_project().model_manager.remove_model(model)
        return model

    def restore(self, context, widget, model):
        context.get_project().model_manager.add_model(model)

# Disable headers-clickable, see bug #163851
prop_registry.override_simple('GtkTreeView::headers-clickable', enabled=False)

class ModelPropEditor(PropertyCustomEditorWithDialog):
    dialog_class = ModelEditorDialog
    button_text = _('Edit Model...')

class ModelAdaptor(CustomProperty, StringType):
    """This represents a fake property to edit the model of a TreeView"""
    custom_editor = ModelPropEditor
    translatable = False

    def get(self):
        model = self.object.get_model()
        if model:
            gzmodel = GazpachoModel.from_model(model)
            return gzmodel.get_name()

    def save(self):
        return self.get()

prop_registry.override_simple('GtkTreeView::model', ModelAdaptor)

class CellsDialogMixin(object):
    """This mixin provides common functionality to dialogs working with
    cell layouts

    It create widgets and add it to self.vbox so this widget should exist in
    the base class (as in dialogs)
    """

    def _make_layout_style(self, group, label, size_group):
        hbox = gtk.HBox()
        hbox.set_spacing(6)
        rb = gtk.RadioButton(group, label)
        size_group.add_widget(rb)
        hbox.pack_start(rb, False, False)
        fr = gtk.Frame()
        eb = gtk.EventBox()
        eb.modify_bg(gtk.STATE_NORMAL, eb.style.white)
        cv = gtk.CellView()
        eb.add(cv)
        fr.add(eb)
        hbox.pack_start(fr, True, True)
        self.vbox.pack_start(hbox, False, False)
        return rb, cv

    def _create_widgets(self):
        title = gtk.Label()
        title.set_markup('<span size="larger" weight="bold">%s</span>' % \
                         _('Layout style'))
        self.vbox.pack_start(title, False, False)

        # with the size group we don't need a table to align the labels on
        # the left column
        size_group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        group = None
        self.text_only, cell = self._make_layout_style(group,
                                                       _("Text only"),
                                                       size_group)
        group = self.text_only
        renderer = gtk.CellRendererText()
        renderer.set_property('markup', 'Text')
        cell.pack_start(renderer)

        self.image_text, cell = self._make_layout_style(group,
                                                        _("Image and text"),
                                                        size_group)
        renderer = gtk.CellRendererPixbuf()
        renderer.set_property('stock-id', gtk.STOCK_OK)
        renderer.set_property('stock-size', gtk.ICON_SIZE_MENU)
        cell.pack_start(renderer, expand=False)
        renderer = gtk.CellRendererText()
        renderer.set_property('markup', 'Text')
        cell.pack_start(renderer)

        self.check_text, cell = self._make_layout_style(group,
                                                        _("Check and text"),
                                                        size_group)
        renderer = gtk.CellRendererToggle()
        renderer.set_property('active', True)
        cell.pack_start(renderer, expand=False)
        renderer = gtk.CellRendererText()
        renderer.set_property('markup', 'Text')
        cell.pack_start(renderer)

        self.custom = gtk.RadioButton(group, _("Custom"))
        size_group.add_widget(self.custom)
        hbox = gtk.HBox()
        hbox.set_spacing(6)
        hbox.pack_start(self.custom, False, False)
        custom_btn = gtk.Button(stock=gtk.STOCK_EDIT)
        custom_btn.set_sensitive(False)
        self.custom.set_sensitive(False)
        hbox.pack_start(custom_btn, True)
        self.vbox.pack_start(hbox, False, False)

        self.vbox.set_size_request(300, -1)

    def get_cell_renderers(self):
        """Returns a list of tuples. Each tuple has a type and a renderer"""
        renderers = []
        if self.text_only.get_active():
            renderers.append((str, gtk.CellRendererText()))
        elif self.image_text.get_active():
            renderers.append((gtk.gdk.Pixbuf, gtk.CellRendererPixbuf()))
            renderers.append((str, gtk.CellRendererText()))
        elif self.check_text.get_active():
            renderers.append((bool, gtk.CellRendererToggle()))
            renderers.append((str, gtk.CellRendererText()))
        else:
            print('custom layout not supported yet')

        return renderers

class CreateCellsDialog(CellsDialogMixin, BaseDialog):
    """Dialog to create the cell layout of a Column"""
    def __init__(self, parent=None):
        BaseDialog.__init__(self, parent, title=_('New layout'),
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))
        self._create_widgets()

        sep = gtk.HSeparator()
        self.vbox.pack_start(sep, False, False)

        label = gtk.Label(_('Title') + ':')
        hbox = gtk.HBox()
        hbox.set_spacing(6)
        hbox.pack_start(label, False, False)

        self.title_entry = gtk.Entry()

        hbox.pack_start(self.title_entry, True, True)
        self.vbox.pack_start(hbox, False, False)

        self.vbox.show_all()

    def get_title(self):
        return self.title_entry.get_text()

    def set_title(self, title):
        self.title_entry.set_text(title)

class EditCellsDialog(CellsDialogMixin, PropertyEditorDialog):
    def _create_widgets(self):
        CellsDialogMixin._create_widgets(self)
        self.text_only.connect('toggled', self._on_button_toggled)
        self.image_text.connect('toggled', self._on_button_toggled)
        self.check_text.connect('toggled', self._on_button_toggled)
        self.custom.connect('toggled', self._on_button_toggled)

    def set_gadget(self, gadget, proxy):
        super(EditCellsDialog, self).set_gadget(gadget, proxy)

        column = self.gadget.widget
        cell_types = list(map(gobject.type_name, column.get_cell_renderers()))
        cell_types.sort()

        if cell_types == ['GtkCellRendererText']:
            self.text_only.set_active(True)
        elif cell_types == ['GtkCellRendererPixbuf', 'GtkCellRendererText']:
            self.image_text.set_active(True)
        elif cell_types == ['GtkCellRendererText', 'GtkCellRendererToggle']:
            self.check_text.set_active(True)
        else:
            self.custom.set_active(True)

    def _on_button_toggled(self, button):
        # TODO: use the proxy and a custom command so the user can undo this
        parent = Gadget.from_widget(self.gadget.widget.get_parent())
        column = self.gadget.widget

        # remove model types for this column
        if parent:
            parent.adaptor.before_change_column_layout(parent, self.gadget)

        column = self.gadget.widget
        self.gadget.adaptor.create_layout(column, self.get_cell_renderers())

        # add model types for this column
        if parent:
            parent.adaptor.after_change_column_layout(parent, self.gadget)

def get_column_button(column):
    # hackish method borrowed from Kiwi
    widget = column.get_widget()
    while not isinstance(widget, gtk.Button) and widget is not None:
        widget = widget.get_parent()
    return widget

def get_column_parent(column):
    button = get_column_button(column)
    if button:
        return button.get_parent()

class CommandAddRemoveTreeViewColumn(FlipFlopCommandMixin, Command):
    """
    Command for adding and removing columns to a tree view.
    """

    def __init__(self, tree_view, column, project, add):
        """
        Initialize the command.

        @param tree_view: the tree_view the column is being added/removed from
        @type tree_view: gazpacho.gadget.Gadget
        @param column: the column that should be added or removed
        @type column: gazpacho.gadget.Gadget
        @param project: the project that the tree_view belongs to
        @type project: gazpacho.project.Project
        @param add: True if the column should be added
        @type add: bool
        """
        FlipFlopCommandMixin.__init__(self, add)

        if add:
            description = _("Add column to tree view '%s'") % tree_view.name
        else:
            description = _("Remove column '%s' from tree view '%s'") % \
                          (column.name, tree_view.name)

        Command.__init__(self, description)

        self._tree_view = tree_view
        self._column = column
        self._project = project

    def _execute_add(self):
        """
        Add the gadgets to the sizegroup. This might mean that the
        sizegroup will be added as well.
        """
        self._tree_view.adaptor.add_child(self._tree_view, self._column)
        self._project.add_widget(self._column.widget, True)
        self._column.select()

    _execute_state1 = _execute_add

    def _execute_remove(self):
        """
        Remove the gadgets from the sizegroup. This might cause the
        sizegroup to be removed as well.
        """
        self._tree_view.adaptor.remove_child(self._tree_view, self._column)
        self._project.remove_widget(self._column.widget)

    _execute_state2 = _execute_remove

class GazpachoTreeViewColumn(gtk.TreeViewColumn):
    """The purpose of this class is make Gazpacho believe a gtk.TreeViewColumn
    is-a gtk.Widget

    Explanation: in GTK+ a gtk.TreeViewColumn inherits from GObject, not from
    gtk.Widget and so Gazpacho does not know how to handle TreeViewColumns.
    From the user point of view a TreeViewColumn is a 'piece' of a TreeView,
    something like a part of the main widget. So he/she wants to be able to
    see it and manipulate it in Gazpacho.

    So what this class does is redirect/delegate all the widget's methods to
    the internal widget a gtk.TreeViewColumn has (e.g. the button). In order
    to get access to that widget we need to call gtk.TreeViewColumn.set_widget
    first and keep it synchronized when removing a column and adding
    it again (undo/redo)
    """
    def __init__(self):
        super(GazpachoTreeViewColumn, self).__init__()
        self.update_widget()
        self.name = None
        self.reset()

    def set_name(self, name):
        self.name = name

    def get_name(self):
        return self.name

    def add_events(self, events):
        self.get_widget().add_events(events)

    def flags(self):
        return self.get_widget().flags()

    def is_ancestor(self, ancestor):
        return self.get_widget().is_ancestor(ancestor)

    def connect(self, name, handler, data=None):
        if name in ('clicked', 'destroy', 'notify'):
            super(GazpachoTreeViewColumn, self).connect(name, handler, data)

    def set_redraw_on_allocate(self, value):
        self.get_widget().set_redraw_on_allocate(value)

    def drag_source_set(self, *args):
        pass

    def drag_dest_set(self, *args):
        pass

    def _get_button(self):
        return get_column_button(self)

    def get_parent(self):
        return get_column_parent(self)

    parent = property(get_parent)

    def get_parent_window(self):
        return self.get_parent().window

    def get_toplevel(self):
        return self.get_widget().get_toplevel()

    def get_window(self):
        return self._get_button().window

    window = property(get_window)

    def get_allocation(self):
        return self._get_button().allocation

    allocation = property(get_allocation)

    def set_title(self, title):
        self.set_property('title', title)
        widget = self.get_widget()
        if widget is None:
            self.update_widget()
        else:
            widget.set_text(title)

    def get_title(self):
        return self.get_property('title')

    def update_widget(self):
        widget = self.get_widget()
        if widget is None:
            title = self.get_property('title')
            label = gtk.Label(title)
            self.set_widget(label)
            label.show()

    def copy(self, other):
        """Copy all properties from 'other' into self"""
        self.name = other.name
        self.set_title(other.get_property('title'))
        for pspec in gobject.list_properties(gtk.TreeViewColumn):
            if pspec.flags & gobject.PARAM_WRITABLE != 0:
                value = other.get_property(pspec.name)
                self.set_property(pspec.name, value)

    def hide(self):
        self.get_widget().hide()

    def show(self):
        self.get_widget().show()

    def show_all(self):
        self.get_widget().show_all()

    def reset(self):
        self.clear()
        self.cell_renderers = []
        self.types = {}
        self.attributes = {}
        self.names = {}

    def add_cell_renderer(self, renderer_type, renderer, **attrs):
        self.cell_renderers.append(renderer)
        self.types[renderer] = renderer_type
        self.attributes[renderer] = attrs
        name = '%s-renderer%d' % (self.name, len(self.cell_renderers))
        self.names[renderer] = name
        self.pack_start(renderer)
        if attrs:
            self.set_attributes(renderer, **attrs)

    def get_type_for_cell_renderer(self, renderer):
        return self.types[renderer]

    def get_name_for_cell_renderer(self, renderer):
        return self.names[renderer]

    def add_attribute(self, renderer, attribute, column):
        self.attributes[renderer][attribute] = column
        super(GazpachoTreeViewColumn, self).add_attribute(renderer, attribute,
                                                          column)

    def get_attribute(self, renderer, attribute):
        return self.attributes[renderer][attribute]

    def get_attributes(self, renderer):
        return self.attributes[renderer]

    def update_attributes(self, shift):
        for r in self.cell_renderers:
            for key, value in list(self.attributes[r].items()):
                self.attributes[r][key] = value + shift

class TreeViewColumnAdaptor(WidgetAdaptor):
    def create(self, context, interactive=True):
        return GazpachoTreeViewColumn()

    def post_create(self, context, column, interactive=True):
        if not interactive:
            return

        d = CreateCellsDialog(context.get_application_window())
        d.set_title(column.get_name())
        response = d.run()
        if response == gtk.RESPONSE_OK:
            column.set_title(d.get_title())
            self.create_layout(column, d.get_cell_renderers())
            d.destroy()
        else:
            d.destroy()
            raise CreationCancelled()

    def create_layout(self, column, cells):
        column.reset()
        for cell_type, cell_renderer in cells:
            attributes = cell_renderer.get_data('gazpacho::attributes') or {}
            column.add_cell_renderer(cell_type, cell_renderer, **attributes)

    def get_children(self, context, column):
        return column.get_cell_renderers()

    def load(self, context, column):
        # column is a gtk.TreeViewColumn and we transform it into a
        # GazpachoTreeViewColumn
        gazpacho_column = GazpachoTreeViewColumn()
        gazpacho_column.copy(column)

        # copy the renderers also
        func = renderer_expert.get_type_based_on_renderer
        renderers = column.get_cell_renderers()
        cells = [(func(gobject.type_name(r)), r) for r in renderers]
        self.create_layout(gazpacho_column, cells)

        # now we replace the pure column with the gazpacho column
        # the old column is not useful anymore so we can set a custom widget
        # just to get its parent. see GTK+ bug #342471
        column.set_widget(gtk.Label())
        tree_view = get_column_parent(column)

        # when pasting a column from the clipboard, it is not attached to a
        # tree view yet
        if tree_view:
            tree_view.remove_column(column)
            tree_view.append_column(gazpacho_column)

        gazpacho_column.update_widget()

        project = context.get_project()
        return Gadget.load(gazpacho_column, project)

class TitleProp(CustomProperty, StringType):
    def get(self):
        return self.object.get_title()

    def set(self, value):
        self.object.set_title(value)

prop_registry.override_simple('GtkTreeViewColumn::title', TitleProp)

class LayoutPropEditor(PropertyCustomEditorWithDialog):
    dialog_class = EditCellsDialog
    button_text = _('Edit Layout Definition...')

class LayoutAdaptor(TransparentProperty, StringType):
    """This represents a fake property to edit the layout of a Column.

    It does not store anything because this information is stored in cell
    renderers inside the column itself. It's just a way to put a button
    in the Gazpacho interface to call the UIPropEditor.
    """
    custom_editor = LayoutPropEditor

prop_registry.override_property('GtkTreeViewColumn::layout', LayoutAdaptor)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Mostly Random WSGI Utilities.

"""
import sys, logging
log = logging.getLogger(__name__)
PY3K = sys.version_info[0] == 3

import socket, hashlib
try:
    #Py3
    from http.client import responses
    from http.cookies import SimpleCookie
except ImportError:
    import platform
    log.debug("%s WSGI environment detected." % platform.python_implementation())
    from httplib import responses
    from Cookie import SimpleCookie

from wsgiref.headers import Headers as ResponseHeaders
from datetime import datetime

try:
    import chardet
except ImportError:
    # charset autodetection disabled
    pass

__all__ = (
        'sniff_content_encoding',
        'format_status_int',
        'IterableWSGIResponse', 
        'HTTPResponse',
        )

def bytearray2str(obj, mode='replace'):
    obj.decode('utf8', errors=mode)
    return obj

def sniff_content_encoding(s, default='utf-8'):
    """
    Attempts to detect the character set of a string value ``s`` or 
    ``default`` if the ``chardet`` module is not found.

    """
    try:
        content_encoding = chardet.detect(s)['encoding']
    except (NameError, UnicodeDecodeError):
        content_encoding = default

    return content_encoding

def format_status_int(status_int):
    """
    Returns a string like "200 OK" if the status_int
    numeric value matches something in our internal
    map.
    
    """
    try:
        status_code = responses[int(status_int)]
    except (TypeError, KeyError):
        status_code = responses[500]    
    
    return "%s %s" % (status_int, status_code)

class IterableWSGIResponse(object):
    """
    A iterable WSGI object using a simple API inspired by Django
    ``HttpResponse``.
    
    >>>response = IterableWSGIResponse(content='hello world', mimetype='text/plain')
    """
    
    # default headers 
    headerClass = ResponseHeaders
    status_int = None
    etag = None

    def __init__(
        self, content='', request=None, status=None, headers=[], 
        mimetype='text/plain', charset='UTF-8', force_unicode=False, 
        cookies=None):
        """Create a new WSGI response instance.
        
        If ``force_unicode`` is set to ``False``, disable explicit
        multibyte conversion. (Binary files handlers may need this)
        """
        self.request = request
        self.mimetype = mimetype
        self.charset = charset
        # XXX workaround the broken unicode stuff in python 2.7.3 
        # note: wsgi require a str though

        if PY3K: # Python 3
            self.content = bytes(content, charset)
        else:    
        	self.content = str(content)

        #assert isinstance(content, bytes), type(content)
                
        # 14.15 - Content-MD5 (for integrity checking of
        # the entity-body)
        self.content_hash = str(
			hashlib.md5(content.encode(charset)).hexdigest()
		)

        # Get the HTTP status code human representation. 
        if status is not None and status >= 200:
            self.status_code = format_status_int(status)
        else:
            # by default attempt to use status_int
            #assert isinstance(self.status_int, int)
            self.status_code = format_status_int(self.status_int)
        
        #update status_int
        if not self.status_int:
            self.status_int = int(self.status_code[0:4])
        
        self.etag = self.content_hash
        
        #if request is not None and request.if_none_match:
        #    self.conditional = True
        #    self.if_none_match = request.environ['HTTP_IF_NONE_MATCH']
        #    #assert request.environ.has_key('HTTP_IF_NONE_MATCH') == True
        #else:
        #    self.conditional = False
        #    self.if_none_match = None

        # Provides a basic HTTP/1.1 headers set
        self.http_headers = [
            ('Content-Type', self.content_type),
            ('Content-Length', self.content_length),
            #('Content-MD5', self.content_hash),
            #('Cache-Control', 'max-age=%i' % (60 * 10)),
            ('Etag', self.etag)
        ]
    
        #if self.conditional:
        #    self.http_headers['ETag'] = self.etag

        if headers is not None:
            for hdr in headers:
                if not hdr in self.http_headers:
                    self.http_headers.append(hdr)
        
        # Handle cookies management
        if cookies is not None:
            for name, value in cookies:
                c = SimpleCookie()
                c[name] = value
                for item in c.itervalues():
                    item['domain'] = socket.getfqdn()
                    #item['path'] = '/'
                    self.http_headers['Set-Cookie'] = item.OutputString()

    def __str__(self, skip_body=False):
        parts = [str(self.status)]
        parts += map('%s: %s'.__mod__, self.headers)
        if not skip_body and self.content != '':
            parts += ['', self.content]
        #outs = bytearray2str(bytearray(parts))
        return '\n'.join(parts)
    
    def __iter__(self):
        """Return a custom iterator type which iterates over the response."""  
        return iter([self.content])

    app_iter = property(__iter__)
    
    #__await__ = __iter__

    def __len__(self):
        """Return the Content-length value (type int)."""
         
        return len(self.content)
    
    def __getitem__(self, k):
        """For foo = response['foo'] operations"""
        try:
            v = self.http_headers[k]
        except KeyError:
            return None
        else:
            return v
    
    def __setitem__(self, item, value):
        """For response['foo'] = 'Bar' operations"""
        try:
            self.http_headers[item] = value
        except:
            raise
            
    content_length = property(__len__)
    
    def __call__(self, environ, start_response):
        if_none_match = environ.get('HTTP_IF_NONE_MATCH', None)
        
        if if_none_match == self.etag:
            #print "request not modified!!!"
            start_response('304 Not Modified', [])
            return []
        else:
            #self.http_headers.add('ETag', self.etag)
            status_code = self.status_code
            start_response(status_code, self.headers)
            #assert isinstance(self.content, bytes)
            return self.app_iter

    def get_content_type(self):
        """Returns the current Content-type header"""
        return "%s; charset=%s" % (self.mimetype, self.charset)
    
    content_type = property(get_content_type)

    def write(self, text=''):
        """Writes characters (text) in the input buffer stream (body)"""
        if isinstance(text, str): 
            text.encode(self.charset)
        self.content += text
    
    def __next__(self):
        """ required method for being a proper iterator type """
        chunk = self.app_iter.__next__()

        #if isinstance(chunk, basestring):
        #    chunk.encode(self.charset)
        return chunk
    next = property(__next__)     
    
    def has_header(self, value):
        """ return True when ``value`` is found in self.headers (HTTP headers)
        """
        try:
            valueof = self.headers[value]
        except (KeyError, TypeError):
            return False
        else:
            return True
    
    @property        
    def headers(self):
        return [(hdr, str(val)) for hdr, val in self.http_headers]

    @property    
    def status(self):
        return self.status_int 

class HTTPResponse(IterableWSGIResponse):
    status_int = 200



from django.core.management.base import NoArgsCommand
import os
import shutil

class Command(NoArgsCommand):
    help = "Copy the satchmo static directory and files to the local project."

    def handle_noargs(self, **options):
        import satchmo_store
        static_src = os.path.join(satchmo_store.__path__[0],'../../static')
        # The static dir could be in a different relative location
        # if satchmo was installed using setup utils
        if not os.path.exists(static_src):
            static_src = os.path.join(satchmo_store.__path__[0],'../static')
        static_dest = os.path.join(os.getcwd(), 'static')
        if os.path.exists(static_dest):
            print "Static directory exists. You must manually copy the files you need."
        else:
            shutil.copytree(static_src, static_dest)
            for root, dirs, files in os.walk(static_dest):
                if '.svn' in dirs:
                    shutil.rmtree(os.path.join(root,'.svn'), True)
            print "Copied %s to %s" % (static_src, static_dest)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2017 Etienne Robillard <tkadm30@yandex.ru>
# All rights reserved.
"""uWSGIController API Version 0.9.1

Middleware for storing uWSGI statistics into the environ object.
"""

import sys
import logging
import demjson
import urllib

logger = logging.getLogger(__name__)

from notmm.controllers.wsgi import WSGIController

__all__ = ('uWSGIController',)


class uWSGIController(WSGIController):

    def __init__(self, wsgi_app=None, settings=None, app_conf=None, stats_url='http://localhost:9001'):
    
        self.wsgi_app = wsgi_app
        self.stats_url = stats_url
        super(uWSGIController, self).__init__(settings=settings, app_conf=app_conf) # hack

    def init_request(self, request):
        #logger.debug('In uWSGIController.init_request...')
        request.environ['uwsgi.requests'] = self.connections(self.stats_url)
        self._request = request
        self._environ = request.environ
        return self._request
    
    def connections(self, url):
        """Return the amount of live connections (requests) for all workers"""
        try:
            fp = urllib.urlopen(url)
        except IOError:
            return 0
        json = demjson.decode(fp.read())

        connections = 0

        for worker in json['workers']:
            connections += worker['requests']
        
        fp.close()

        return connections




import re

##################################################
## SLUG HELPERS ##

_REMOVELIST = [
    "a", "an", "as", "at", "before", "but", "by", "for", "from",
    "is", "in", "into", "like", "of", "off", "on", "onto", "per",
    "since", "than", "the", "this", "that", "to", "up", "via",
    "with"
    ]
_RE_REMOVE = re.compile('\\b(' + '|'.join(_REMOVELIST) + ')\\b', re.IGNORECASE)
_RE_UNNEEDED = re.compile(r'[^\w\s-]')


def URLify(s, num_chars=None):
    """ based on urlify.js from django
        effect:
            removes common words (_REMOVELIST)
            removes unneeded whitespaces around '-'
            trim leading/trailing whitespaces
            lowercase string
            replace space with '-'
            if num_chars if given trim to number of given characters
            finnaly quotes other URLunsafe words with urlib.quote
    """

    ret = _RE_REMOVE.sub('', s)
    ret = _RE_UNNEEDED.sub('', s)
    ret = ret.strip().replace(' ', '-').lower()

    if num_chars is not None:
        ret = ret[:num_chars]

    # remove duplicate '-'
    while '--' in ret: ret = ret.replace('--', '-')

    return ret
# URLify

_BR_CLEANUP = re.compile(r'<\s*br\s*/?\s*>', re.IGNORECASE)

def text_cleanup(txt, clean_br=True, cleanup_double_space=True, strip_ws=True):
    if txt:
        if clean_br:
            txt = _BR_CLEANUP.sub(' ', txt) # remove <br />
        #
        
        if strip_ws:
            txt = txt.strip() # strip all leading/trailing spaces
        #
        
        if cleanup_double_space:
            while '  ' in txt: txt = txt.replace('  ', ' ') # remove double spaces
        #

        return txt
    else:
        return txt
#



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models

class Question(models.Model):
    question_text = models.CharField(max_length=200)
    pub_date = models.DateTimeField('date published')

class Choice(models.Model):
    question = models.ForeignKey(Question, on_delete=models.CASCADE)
    choice_text = models.CharField(max_length=200)
    votes = models.IntegerField(default=0)



# Copyright (C) 2007-2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
#
# Please see the "LICENSE" and "NOTICE" files included in the source 
# distribution for details on licensing info.

from schevo.schema import *
from schevo.constant import UNASSIGNED
schevo.schema.prep(locals())

from notmm.utils.decorators import permalink
from markdown2 import markdown, MarkdownError

class SchevoIcon(E.Entity):

    _hidden = True

    name = f.string()
    data = f.image()

    _key(name)

#class Event(E.Entity):
#    title = f.string()
#    _plural = "Events"

class Category(E.Entity):
    name = f.string()
    description = f.string(required=False)
    slug = f.string(required=False)

    _key(name)
    _plural = "Categories"

    def __str__(self):
        return "%s" % self.name

    def __repr__(self):
        return self.__str__()

    #def as_slug(self):
    #    if self.slug:
    #        slug = self.slug
    #    else:
    #        slug = self.name.replace(' ', '-').lower()
    #    return str(slug)

    def x_category_items(self):
        return [x for x in self.m.category_items()]
   
    def x_blogentries(self):
        from operator import attrgetter
        items = sorted([item for item in self.m.blogentries() if \
            item.published is True and type(item) != UNASSIGNED], \
            key=attrgetter('pub_date'), reverse=True)
        return items
 
    def get_items_count(self):
        rv = []
        for item in self.x.blogentries():
            rv.append(item)
        return len(rv)

class CategoryItem(E.Entity):
    name = f.string()
    category = f.entity('Category')
    description = f.string(required=False)
    slug = f.string(required=False)
    _plural = "Category items"

    def __str__(self):
        return 'CategoryItem: %s' % self.name
    
    #def x_blogentries(self):
    #    from operator import attrgetter
    #    items = sorted([item for item in self.m.blogentries() if \
    #        item.published is True and type(item) != UNASSIGNED], \
    #        key=attrgetter('pub_date'), reverse=True)
    #    return items

class Comment(E.Entity):
    
    #sender_name = f.entity('Author')
    sender_name = f.string()
    sender_message = f.string(multiline=True)
    sender_email = f.string()
    sender_website = f.string(required=False, allow_empty=True)
    
    #TODO support more flexible content/object types...
    blogentry = f.entity('BlogEntry')

    published = f.boolean(required=False, default=False)
    pub_date = f.datetime()

    #_key(sender_name, blogentry)
    #_key(blogentry)

    def x_is_published(self):
        return (self.published == True)

    ### Public methods
    def convert_to_html(self, name='sender_message'):
        # convert a field value to markdown
        try:
            # Can only convert strings, not UNASSIGNED or None fields.. 
            from notmm.utils.markup import convert_to_markdown
            html = convert_to_markdown(getattr(self, name))
        except (AttributeError, MarkdownError), e:
            raise e
        else:
            return html

##XXX: Uses tm.accounts.model.UserManager
class Author(E.Entity):
    #user = f.entity
    username = f.string()
    # TODO: add a emailstring() method to support e-mail bytestrings in utf8
    email = f.string(required=False)

    homepage_url = f.string(required=False)
    blog_url = f.string(required=False)
    
    #twitter_username = f.string(required=False)

    _key(username, email)

    #def x_blogentries(self):
    #    return [casting.movie
    #            for casting in self.m.movie_castings()]
    
    def __str__(self):
        return self.username

class Tag(E.Entity):

    name = f.string()
    blogentry = f.entity('BlogEntry')
    #description = f.string(required=False)

    _key(name, blogentry)

#class Article
class BlogEntry(E.Entity):

    title = f.string()
    pub_date = f.datetime()
    author = f.entity('Author')
    
    category = f.entity('Category')
    #categoryitem = f.entity('CategoryItem', required=False)
    short_description = f.string(multiline=False, required=False, default='')
    
    # TODO: add markdown or textile support here
    body = f.string(multiline=True, required=False)
    
    # /path/to/article.md
    source = f.path(required=False)
    
    #Need a f.multiplechoices field type!
    #source_type = f.string(
    #    preferred_values=('rest', 'html', 'text'),
    #    required=False)
    
    reviewed = f.boolean(required=False, default=False, was='published')
    slug = f.string(required=True)

    _key(title)
    _plural = "BlogEntries"

    #def x_category_items(self):
    #    return [item for item in self.m.categoryitems()]

    def x_comments(self):
        return [comment for comment in self.m.comments()]

    def x_tags(self):
        return [tag for tag in self.m.tags()]

    def __str__(self):
        return '%s: %s' % (self.author, self.title)

    #### public methods

    @permalink
    def get_absolute_url(self):
        # returns the canonical URL representation for this post
        assert type(self.category) != UNASSIGNED
        category_slug = self.category.slug
        return ('blogengine.server.views.details_for_blogentry', (), dict(
            category=category_slug, 
            slug=self.slug))
    
    def convert_to_html(self, name='body'):
        """ convert to html with markdown """
        # XXX: support other wiki-like converters
        try:
            # Can only convert strings, not _UNASSIGNED or None fields.. 
            from notmm.utils.markup import convert_to_markdown
            if getattr(self, name) is not UNASSIGNED:
                html = convert_to_markdown(getattr(self, name))
        except AttributeError, e:
            raise e
        else:
            return html

#E.Comment._sample = [
#    (('Etienne Robillard',), ('Pepe is not a pony!',), 
#      'this is a test comment!', True, '2009-05-17 12:54'),
#]
#E.Tag._sample = [
#    ('notmm', ('Pepe is not a pony!',)),
#] 
#E.BlogEntry._sample = [
#    ('This is a test using notmm-schevo', '2009-05-04', 
#    ('Etienne Robillard', ), 'Just a test', 'Long description here'),
#    ('2nd post!', '2009-05-04',
#    ('Etienne Robillard', ), '2nd test with notmm-schevo', 'so far so good :)')]




import keyedcache
import logging

log = logging.getLogger('keyedcache')

class CachedObjectMixin(object):
    """Provides basic object keyedcache for any objects using this as a mixin."""

    def cache_delete(self, *args, **kwargs):
        key = self.cache_key(*args, **kwargs)
        log.debug("clearing cache for %s", key)
        keyedcache.cache_delete(key, children=True)

    def cache_get(self, *args, **kwargs):
        key = self.cache_key(*args, **kwargs)
        return keyedcache.cache_get(key)

    def cache_key(self, *args, **kwargs):
        keys = [self.__class__.__name__, self]
        keys.extend(args)
        return keyedcache.cache_key(keys, **kwargs)

    def cache_reset(self):
        self.cache_delete()
        self.cache_set()

    def cache_set(self, *args, **kwargs):
        val = kwargs.pop('value', self)
        key = self.cache_key(*args, **kwargs)
        keyedcache.cache_set(key, value=val)
        
    def is_cached(self, *args, **kwargs):
        return keyedcache.is_cached(self.cache_key(*args, **kwargs))
        
def find_by_id(cls, groupkey, objectid, raises=False):
    """A helper function to look up an object by id"""
    ob = None
    try:
        ob = keyedcache.cache_get(groupkey, objectid)
    except keyedcache.NotCachedError, e:
        try: 
            ob = cls.objects.get(pk=objectid)
            keyedcache.cache_set(e.key, value=ob)

        except cls.DoesNotExist:
            log.debug("No such %s: %s", groupkey, objectid)
            if raises:
                raise cls.DoesNotExist

    return ob


def find_by_key(cls, groupkey, key, raises=False):
    """A helper function to look up an object by key"""
    ob = None
    try:
        ob = keyedcache.cache_get(groupkey, key)
    except keyedcache.NotCachedError, e:
        try: 
            ob = cls.objects.get(key__exact=key)
            keyedcache.cache_set(e.key, value=ob)

        except cls.DoesNotExist:
            log.debug("No such %s: %s", groupkey, key)
            if raises:
                raise

    return ob
    
def find_by_slug(cls, groupkey, slug, raises=False):
    """A helper function to look up an object by slug"""
    ob = None
    try:
        ob = keyedcache.cache_get(groupkey, slug)
    except keyedcache.NotCachedError, e:
        try: 
            ob = cls.objects.get(slug__exact=slug)
            keyedcache.cache_set(e.key, value=ob)

        except cls.DoesNotExist:
            log.debug("No such %s: %s", groupkey, slug)
            if raises:
                raise

    return ob




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

from product.migrations import UpdateContentTypeMigration

class Migration(UpdateContentTypeMigration):

    def forwards(self, orm):
        self.migrate_contenttype('product', 'downloadable', ('downloadableproduct', ))
        self.migrate_contenttype('shop', 'downloadable', ('downloadlink', ))

    def backwards(self, orm):
        self.migrate_contenttype('downloadable', 'product', ('downloadableproduct', ))
        self.migrate_contenttype('downloadable', 'shop', ('downloadlink', ))

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True'})
        },
        'contact.contact': {
            'Meta': {'object_name': 'Contact'},
            'create_date': ('django.db.models.fields.DateField', [], {}),
            'dob': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'notes': ('django.db.models.fields.TextField', [], {'max_length': '500', 'blank': 'True'}),
            'organization': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contact.Organization']", 'null': 'True', 'blank': 'True'}),
            'role': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contact.ContactRole']", 'null': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True', 'null': 'True', 'blank': 'True'})
        },
        'contact.contactorganization': {
            'Meta': {'object_name': 'ContactOrganization'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'contact.contactorganizationrole': {
            'Meta': {'object_name': 'ContactOrganizationRole'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'contact.contactrole': {
            'Meta': {'object_name': 'ContactRole'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'contact.organization': {
            'Meta': {'object_name': 'Organization'},
            'create_date': ('django.db.models.fields.DateField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'notes': ('django.db.models.fields.TextField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'role': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contact.ContactOrganizationRole']", 'null': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contact.ContactOrganization']", 'null': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'downloadable.downloadableproduct': {
            'Meta': {'object_name': 'DownloadableProduct'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'expire_minutes': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'num_allowed_downloads': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'product': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['product.Product']", 'unique': 'True', 'primary_key': 'True'})
        },
        'downloadable.downloadlink': {
            'Meta': {'object_name': 'DownloadLink'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'downloadable_product': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['downloadable.DownloadableProduct']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'num_attempts': ('django.db.models.fields.IntegerField', [], {}),
            'order': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['shop.Order']"}),
            'time_stamp': ('django.db.models.fields.DateTimeField', [], {})
        },
        'product.category': {
            'Meta': {'unique_together': "(('site', 'slug'),)", 'object_name': 'Category'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'child'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'related_categories': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_categories'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Category']"}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'product.product': {
            'Meta': {'unique_together': "(('site', 'sku'), ('site', 'slug'))", 'object_name': 'Product'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'also_purchased': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'also_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'category': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['product.Category']", 'symmetrical': 'False', 'blank': 'True'}),
            'date_added': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'featured': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'height': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'height_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'items_in_stock': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'length': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'length_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'meta': ('django.db.models.fields.TextField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'ordering': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'related_items': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'related_products'", 'blank': 'True', 'null': 'True', 'to': "orm['product.Product']"}),
            'shipclass': ('django.db.models.fields.CharField', [], {'default': "'DEFAULT'", 'max_length': '10'}),
            'short_description': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'sku': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'taxClass': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['product.TaxClass']", 'null': 'True', 'blank': 'True'}),
            'taxable': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'total_sold': ('django.db.models.fields.DecimalField', [], {'default': "'0'", 'max_digits': '18', 'decimal_places': '6'}),
            'weight': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '8', 'decimal_places': '2', 'blank': 'True'}),
            'weight_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'}),
            'width': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '6', 'decimal_places': '2', 'blank': 'True'}),
            'width_units': ('django.db.models.fields.CharField', [], {'max_length': '3', 'null': 'True', 'blank': 'True'})
        },
        'product.taxclass': {
            'Meta': {'object_name': 'TaxClass'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '30'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        'shop.order': {
            'Meta': {'object_name': 'Order'},
            'bill_addressee': ('django.db.models.fields.CharField', [], {'max_length': '61', 'blank': 'True'}),
            'bill_city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'bill_country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'bill_postal_code': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'bill_state': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'bill_street1': ('django.db.models.fields.CharField', [], {'max_length': '80', 'blank': 'True'}),
            'bill_street2': ('django.db.models.fields.CharField', [], {'max_length': '80', 'blank': 'True'}),
            'contact': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contact.Contact']"}),
            'discount': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'}),
            'discount_code': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'method': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'ship_addressee': ('django.db.models.fields.CharField', [], {'max_length': '61', 'blank': 'True'}),
            'ship_city': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'ship_country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'ship_postal_code': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'ship_state': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'ship_street1': ('django.db.models.fields.CharField', [], {'max_length': '80', 'blank': 'True'}),
            'ship_street2': ('django.db.models.fields.CharField', [], {'max_length': '80', 'blank': 'True'}),
            'shipping_cost': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'}),
            'shipping_description': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'shipping_discount': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'}),
            'shipping_method': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'shipping_model': ('shipping.fields.ShippingChoiceCharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'sub_total': ('satchmo_utils.fields.CurrencyField', [], {'display_decimal': '4', 'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'}),
            'tax': ('satchmo_utils.fields.CurrencyField', [], {'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'}),
            'time_stamp': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'total': ('satchmo_utils.fields.CurrencyField', [], {'display_decimal': '4', 'null': 'True', 'max_digits': '18', 'decimal_places': '10', 'blank': 'True'})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['downloadable']



from django.conf.urls.defaults import *

urlpatterns = patterns('livesettings.views',
    (r'^$', 'site_settings', {}, 'satchmo_site_settings'),
    (r'^export/$', 'export_as_python', {}, 'settings_export'),
    (r'^(?P<group>[^/]+)/$', 'group_settings'),
)



####################################################################
# First step in the order process - capture all the demographic info
#####################################################################

from django import http
from django.core import urlresolvers
from django.shortcuts import render_to_response
from django.template import RequestContext
from livesettings import config_get_group, config_value
from satchmo_store.contact import CUSTOMER_ID
from satchmo_store.contact.models import Contact
from payment.decorators import cart_has_minimum_order
from payment.forms import PaymentContactInfoForm
from satchmo_store.shop.models import Cart, Config, Order
from satchmo_utils.dynamic import lookup_url
from signals_ahoy.signals import form_initialdata

import logging

log = logging.getLogger('satchmo_store.contact.contact')

def authentication_required(request, template='shop/checkout/authentication_required.html'):
    return render_to_response(
        template, {}, context_instance = RequestContext(request)
    )

def contact_info(request, **kwargs):
    """View which collects demographic information from customer."""

    #First verify that the cart exists and has items
    tempCart = Cart.objects.from_request(request)
    if tempCart.numItems == 0:
        return render_to_response('shop/checkout/empty_cart.html',
                                  context_instance=RequestContext(request))

    if not request.user.is_authenticated() and config_value('SHOP', 'AUTHENTICATION_REQUIRED'):
        url = urlresolvers.reverse('satchmo_checkout_auth_required')
        thisurl = urlresolvers.reverse('satchmo_checkout-step1')
        return http.HttpResponseRedirect(url + "?next=" + thisurl)

    init_data = {}
    shop = Config.objects.get_current()
    if request.user.is_authenticated():
        if request.user.email:
            init_data['email'] = request.user.email
        if request.user.first_name:
            init_data['first_name'] = request.user.first_name
        if request.user.last_name:
            init_data['last_name'] = request.user.last_name
    try:
        contact = Contact.objects.from_request(request, create=False)
    except Contact.DoesNotExist:
        contact = None
        
    try:
        order = Order.objects.from_request(request)
        if order.discount_code:
            init_data['discount'] = order.discount_code
    except Order.DoesNotExist:
        pass

    if request.method == "POST":
        new_data = request.POST.copy()
        if not tempCart.is_shippable:
            new_data['copy_address'] = True
        form = PaymentContactInfoForm(data=new_data, shop=shop, contact=contact, shippable=tempCart.is_shippable, 
            initial=init_data, cart=tempCart)

        if form.is_valid():
            if contact is None and request.user and request.user.is_authenticated():
                contact = Contact(user=request.user)
            custID = form.save(request, cart=tempCart, contact=contact)
            request.session[CUSTOMER_ID] = custID
                        
            modulename = new_data['paymentmethod']
            if not modulename.startswith('PAYMENT_'):
                modulename = 'PAYMENT_' + modulename
            paymentmodule = config_get_group(modulename)
            url = lookup_url(paymentmodule, 'satchmo_checkout-step2')
            return http.HttpResponseRedirect(url)
        else:
            log.debug("Form errors: %s", form.errors)
    else:
        if contact:
            #If a person has their contact info, make sure we populate it in the form
            for item in contact.__dict__.keys():
                init_data[item] = getattr(contact,item)
            if contact.shipping_address:
                for item in contact.shipping_address.__dict__.keys():
                    init_data["ship_"+item] = getattr(contact.shipping_address,item)
            if contact.billing_address:
                for item in contact.billing_address.__dict__.keys():
                    init_data[item] = getattr(contact.billing_address,item)
            if contact.primary_phone:
                init_data['phone'] = contact.primary_phone.phone
        else:
            # Allow them to login from this page.
            request.session.set_test_cookie()

        #Request additional init_data
        form_initialdata.send(sender=PaymentContactInfoForm, initial=init_data, 
            contact=contact, cart=tempCart, shop=shop)

        form = PaymentContactInfoForm(
            shop=shop, 
            contact=contact, 
            shippable=tempCart.is_shippable, 
            initial=init_data, 
            cart=tempCart)

    if shop.in_country_only:
        only_country = shop.sales_country
    else:
        only_country = None
        
    context = RequestContext(request, {
        'form': form,
        'country': only_country,
        'paymentmethod_ct': len(form.fields['paymentmethod'].choices)
        })
    return render_to_response('shop/checkout/form.html',
                              context_instance=context)

contact_info_view = cart_has_minimum_order()(contact_info)



# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gobject
import gtk

from gazpacho.i18n import _
from gazpacho.propertyeditor import PropertyEditorDialog
from gazpacho.util import select_iter

def create_image_only_button(stock, size=gtk.ICON_SIZE_MENU):
    """Create a stock button with only image"""
    image = gtk.Image()
    image.set_from_stock(stock, size)
    button = gtk.Button()
    button.add(image)
    return button

def create_scrolled_window(child,
                           hpolicy=gtk.POLICY_AUTOMATIC,
                           vpolicy=gtk.POLICY_AUTOMATIC):
    """Helper function to avoid repetitive code"""
    scrolled_window = gtk.ScrolledWindow()
    scrolled_window.set_shadow_type(gtk.SHADOW_IN)
    scrolled_window.set_policy(hpolicy, vpolicy)
    scrolled_window.add(child)
    return scrolled_window

class RendererExpert(object):
    """This singletone knows everything you always wanted to know about cell
    renderers

    - It knows which renderers is best for a data type
    - It knows how to edit a cell used by a particular renderer
    - It knows the best column title based on the renderer that column uses
    - It knows

    Ask him whatever you need about cell renderers. He knows everything!
    """

    # for each type we have:
    #  - renderer that best represent that type
    #  - title for a tree view column
    #  - renderer that's best for editing a model attribute of that type
    #  - renderer attribute that should be connected to the model data
    knowledge = {
        str: (gtk.CellRendererText, 'string',
              gtk.CellRendererText, 'text'),
        bool: (gtk.CellRendererToggle, 'bool',
               gtk.CellRendererToggle, 'active'),
        int: (gtk.CellRendererText, 'float',
              gtk.CellRendererText, 'text'),
        float: (gtk.CellRendererText, 'float',
                gtk.CellRendererText, 'text'),
        gtk.gdk.Pixbuf: (gtk.CellRendererPixbuf, 'image',
                         gtk.CellRendererText, 'pixbuf'),
        }

    def get_renderer_for_type(self, t):
        """What's the best renderer for editing a column of type t?"""
        return self.knowledge[t][2]()

    def get_column_name_for_type(self, t):
        """What's the best column title for a column of type t?"""
        return self.knowledge[t][1]

    def get_type_based_on_renderer(self, r):
        """If we already have a renderer, what should be its best type?"""
        for t, data in list(self.knowledge.items()):
            if gobject.type_name(data[0]) == r:
                return t

    def get_attribute_for_model(self, t):
        """What attribute of the renderer should be set with the model data?"""
        return self.knowledge[t][3]

    def get_type_names(self):
        """Returns a list with all the possible type names we know about"""
        return [v[1] for v in list(self.knowledge.values())]

    def make_renderer_editable(self, t, renderer, model, column):
        "Make this renderer editable and save its interactions into the model"
        if t == bool:
            renderer.set_property('activatable', True)
            renderer.connect('toggled', self._on_cell_toggled, model, column)
        else:
            renderer.set_property('editable', True)
            renderer.connect('edited', self._on_cell_edited, model, column)

    def _on_cell_edited(self, cell, path, new_text, model, column):
        model[path][column] = new_text

    def _on_cell_toggled(self, cell, path, model, column):
        model[path][column] = not model[path][column]

renderer_expert = RendererExpert()

GAZPACHO_MODEL_QDATA = 'gazpacho::model'

class GazpachoModel(object):
    """Wrapper around a TreeModel"""
    # mapping from gtypes to python types. Used when loading a model
    gtypes2python = {
        gobject.TYPE_STRING: str,
        gobject.TYPE_BOOLEAN: bool,
        gobject.TYPE_INT: int,
        gobject.TYPE_FLOAT: float,
        }
    def __init__(self, name, model=None):
        self._name = name

        self._hierarchical = False

        if model is not None:
            # load the given model
            n = model.get_n_columns()
            gtypes = [model.get_column_type(i) for i in range(n)]
            self._types = []
            for t in gtypes:
                try:
                    self._types.append(self.gtypes2python[t])
                except KeyError:
                    self._types.append(t)

            self._types += [str]
            self._model = gtk.ListStore(*self._types)
            self._copy_models(model, self._model)
        else:
            # the initial type is a fake one just to make the TreeView
            # shows its header and hence, the '+' button
            self._model = gtk.ListStore(str)

            self._types = [str]

        self._model.set_data(GAZPACHO_MODEL_QDATA, self)

    #@classmethod
    def from_model(cls, widget):
        return widget.get_data(GAZPACHO_MODEL_QDATA)
    from_model = classmethod(from_model)

    def get_name(self):
        return self._name

    def set_name(self, name):
        self._name = name

    def add_column(self, column_type):
        """Add the column to the model and returns its index"""
        self._types = self._types[:-1]
        preserve_columns = []
        if len(self._types) > 0:
            preserve_columns = list(range(len(self._types)))
        self._types.append(column_type)
        self._types += [str]

        self._update_model(preserve_columns)

        return len(self._types) - 2

    def remove_column(self, column_index):
        preserve_columns = list(range(len(self._types)))
        del self._types[column_index]
        del preserve_columns[column_index]

        self._update_model(preserve_columns, column_index)

    def add_columns_for_tree_view_column(self, column):
        for renderer in column.get_cell_renderers():
            t = column.get_type_for_cell_renderer(renderer)
            index = self.add_column(t)
            attribute = renderer_expert.get_attribute_for_model(t)

            column.add_attribute(renderer, attribute, index)

    def remove_columns_of_tree_view_column(self, column):
        count = 0
        for renderer in column.get_cell_renderers():
            t = column.get_type_for_cell_renderer(renderer)
            attribute = renderer_expert.get_attribute_for_model(t)

            index = column.get_attribute(renderer, attribute)

            self.remove_column(index - count)
            count += 1

    def insert_columns_of_tree_view_column(self, column, index):
        offset = index
        preserve_columns = list(range(len(self._types)))
        ignore_columns = []
        for renderer in column.get_cell_renderers():
            t = column.get_type_for_cell_renderer(renderer)
            self._types.insert(offset, t)

            attribute = renderer_expert.get_attribute_for_model(t)
            column.add_attribute(renderer, attribute, offset)

            ignore_columns.append(offset)
            offset += 1

        self._update_model(preserve_columns, -1, ignore_columns)

    def get_hierarchical(self):
        return self._hierarchical

    def set_hierarchical(self, value):
        self._hierarchical = value

    def get_model(self):
        return self._model

    def get_types(self):
        # the last one is fake
        return self._types[:-1]

    def get_type_name(self):
        return gobject.type_name(self._model)

    def get_column_type_names(self):
        """
        @returns: a list of type names
        """
        model = self._model
        return [gobject.type_name(model.get_column_type(i))
                    for i in range(model.get_n_columns() - 1)]

    # Private

    def _update_model(self, preserve_columns=tuple(), removed_column=-1,
                      ignore_columns=tuple()):
        if preserve_columns:
            data = self._backup_data(preserve_columns)

        if self._hierarchical:
            self._model = gtk.TreeStore(*self._types)
        else:
            self._model = gtk.ListStore(*self._types)

        if preserve_columns:
            self._restore_data(data, removed_column, ignore_columns)

        self._model.set_data(GAZPACHO_MODEL_QDATA, self)

    def _backup_data(self, columns):
        data = []
        for row in self._model:
            data.append({})
            for col in columns:
                data[-1][col] = row[col]
        return data

    def _restore_data(self, data, removed_column=-1, ignore_columns=tuple()):
        for d in data:
            iter = self._model.append()
            for col, value in list(d.items()):
                if col in ignore_columns:
                    col += len(ignore_columns)
                if removed_column != -1 and col > removed_column:
                    col -= 1
                self._model.set_value(iter, col, value)

    def _copy_models(self, src, dst):
        """Copy the data from src to dst"""
        dst.clear()
        for row in src:
            iter = dst.append()
            for i, value in enumerate(row):
                dst.set_value(iter, i, value)

class GazpachoModelManager(object):
    def __init__(self):
        self._models = {}

    def _create_new_model_name(self):
        name = None
        i = 0
        while True:
            i += 1
            name = "model%d" % i
            if not name in self._models:
                break

        return name

    def create_model(self):
        """Add a model to the list using a unique name"""
        name = self._create_new_model_name()
        model = self._models[name] = GazpachoModel(name)
        return model

    def get_model(self, name):
        return self._models[name]

    def get_models(self):
        return list(self._models.values())

    def load_model(self, model):
        self._models[model.name] = GazpachoModel(model.name, model)

    def remove_model(self, model):
        del self._models[model.get_name()]

    def add_model(self, model):
        # check if we have a model with the same name
        name = model.get_name()
        if name in list(self._models.keys()):
            # give the model a new name
            name = self._create_new_model_name()
            model.set_name(name)

        self._models[name] = model

(CUSTOM_MODEL_TYPE,
 CUSTOM_MODEL_INDEX,
 CUSTOM_MODEL_TYPE_NAME) = list(range(3))

class ModelEditorDialog(PropertyEditorDialog):

    def _create_widgets(self):
        title = gtk.Label()
        title.set_markup('<span size="larger" weight="bold">%s</span>' %
                         _('Model editor'))
        self.vbox.pack_start(title, False, False)

        self.tabs = gtk.Notebook()
        self.vbox.pack_start(self.tabs, True, True)

        data_tab = gtk.VBox()
        self._create_data_tab(data_tab)
        self.tabs.append_page(data_tab, gtk.Label(_('Data')))

        types_tab = gtk.VBox()
        self._create_types_tab(types_tab)
        self.tabs.append_page(types_tab, gtk.Label(_('Types')))

        self.vbox.set_size_request(360, 340)

    def _create_types_tab(self, tab):
        tab.set_border_width(6)
        tab.set_spacing(6)

        text = _('Create model from widget')
        self.create_from_widget_btn = gtk.RadioButton(None, text)

        self.create_from_widget_btn.connect('toggled',
                                            self._on_creation_type_toggled)
        hbox = gtk.HBox()
        hbox.set_spacing(6)

        hbox.pack_start(self.create_from_widget_btn, False, False)
        self.widget_combo = gtk.combo_box_new_text()
        hbox.pack_start(self.widget_combo)
        tab.pack_start(hbox, False, False)

        self.custom_model_btn = gtk.RadioButton(self.create_from_widget_btn,
                                                _('Custom model'))
        self.custom_model_btn.connect('toggled',
                                      self._on_creation_type_toggled)
        tab.pack_start(self.custom_model_btn, False, False)

        self.custom_model_align = gtk.Alignment(0, 0, 1, 1)
        self.custom_model_align.set_padding(0, 0, 48, 0)

        hbox = gtk.HBox()
        hbox.set_spacing(6)
        self.custom_model_store = gtk.ListStore(object, int, str)
        self.custom_model_view = gtk.TreeView(self.custom_model_store)
        renderer1 = gtk.CellRendererText()
        renderer1.set_property('xalign', 1) # right alignment
        col1 = gtk.TreeViewColumn(_('Index'), renderer1,
                                  text=CUSTOM_MODEL_INDEX)
        self.custom_model_view.append_column(col1)
        renderer2 = gtk.CellRendererText()
        col2 = gtk.TreeViewColumn(_('Type'), renderer2,
                                  text=CUSTOM_MODEL_TYPE_NAME)
        self.custom_model_view.append_column(col2)
        scrolled_window = create_scrolled_window(self.custom_model_view,
                                                 hpolicy=gtk.POLICY_NEVER)
        hbox.pack_start(scrolled_window, True, True)

        table = gtk.Table(2, 2)
        self.types_combo = gtk.combo_box_new_text()
        for t in renderer_expert.get_type_names():
            self.types_combo.append_text(t)
        self.types_combo.set_active(0)
        self.add_type_btn = create_image_only_button(gtk.STOCK_ADD)
        self.del_type_btn = create_image_only_button(gtk.STOCK_REMOVE)

        table.attach(self.types_combo, 0, 1, 0, 1, yoptions=0)
        table.attach(self.add_type_btn, 1, 2, 0, 1, yoptions=0)
        table.attach(self.del_type_btn, 1, 2, 1, 2, yoptions=0)
        hbox.pack_start(table, False, False)

        self.custom_model_align.add(hbox)
        self.custom_model_align.set_sensitive(False)
        tab.pack_start(self.custom_model_align, True, True)

    def _create_data_tab(self, tab):
        tab.set_border_width(6)
        tab.set_spacing(6)

        self.tree_store_check = gtk.CheckButton(_('Allow hierarchical data'))
        self.tree_store_check.set_sensitive(False)
        tab.pack_start(self.tree_store_check, False, False)

        self.data_view = gtk.TreeView()
        selection = self.data_view.get_selection()
        selection.connect('changed', self._on_data_view_selection_changed)
        scrolled_window = create_scrolled_window(self.data_view)
        tab.pack_start(scrolled_window, True, True)

        hbox = gtk.HBox()
        table = gtk.Table(2, 3)
        self.add_row_btn = create_image_only_button(gtk.STOCK_ADD)
        self.add_row_btn.connect('clicked', self._on_add_row)
        table.attach(self.add_row_btn, 0, 1, 0, 1, xoptions=0, yoptions=0)
        self.del_row_btn = create_image_only_button(gtk.STOCK_REMOVE)
        self.del_row_btn.connect('clicked', self._on_del_row)
        table.attach(self.del_row_btn, 0, 1, 1, 2, xoptions=0, yoptions=0)
        self.up_row_btn = create_image_only_button(gtk.STOCK_GO_UP)
        self.up_row_btn.connect('clicked', self._on_up_row)
        table.attach(self.up_row_btn, 1, 2, 0, 1, xoptions=0, yoptions=0)
        self.down_row_btn = create_image_only_button(gtk.STOCK_GO_DOWN)
        self.down_row_btn.connect('clicked', self._on_down_row)
        table.attach(self.down_row_btn, 1, 2, 1, 2, xoptions=0, yoptions=0)
        self.left_row_btn = create_image_only_button(gtk.STOCK_GO_FORWARD)
        self.left_row_btn.connect('clicked', self._on_left_row)
        table.attach(self.left_row_btn, 2, 3, 0, 1, xoptions=0, yoptions=0)
        self.right_row_btn = create_image_only_button(gtk.STOCK_GO_BACK)
        self.right_row_btn.connect('clicked', self._on_right_row)
        table.attach(self.right_row_btn, 2, 3, 1, 2, xoptions=0, yoptions=0)

        hbox.pack_start(table, False, False)

        label = gtk.Label(_('Click on a cell to edit its contents'))
        hbox.pack_start(label, True, True)

        tab.pack_start(hbox, False, False)

    def _fill_widget_combo(self, project):
        """Fill the widget combo with the names of all the widgets in this
        project that can have a model"""
        self.widget_combo.get_model().clear()
        model_widgets = (gtk.TreeView, gtk.ComboBox, gtk.ComboBoxEntry)
        widgets = [w for w in project.get_widgets()
                      if isinstance(w, model_widgets)]
        for widget in widgets:
            self.widget_combo.append_text(widget.name)

    def _select_widget_in_widget_combo(self, widget_name):
        """Select the widget_name item in the widget_combo"""
        model = self.widget_combo.get_model()
        for i, row in enumerate(model):
            if row[0] == widget_name:
                self.widget_combo.set_active(i)
                break

    def _synchronize_types_view(self):
        """Fill the custom model with the information of the current model

        This way, if the user chooses 'custom model' he/she does not need to
        start from scratch
        """
        self.custom_model_store.clear()
        for col, t in enumerate(self.model.get_types()):
            name = renderer_expert.get_column_name_for_type(t)
            self.custom_model_store.append((t, col, name))

    def _synchronize_data_view(self):
        """Sync our data treeview with the model we are editing:

        - Put one tree column for each model column
        - Set the model of the treeview to this model
        """
        # clear previous columns
        col = self.data_view.get_column(0)
        while col is not None:
            self.data_view.remove_column(col)
            col = self.data_view.get_column(0)

        # set the new model
        self.data_view.set_model(self.model.get_model())

        # create the columns
        for col, t in enumerate(self.model.get_types()):
            renderer = renderer_expert.get_renderer_for_type(t)
            name = renderer_expert.get_column_name_for_type(t)
            attribute = renderer_expert.get_attribute_for_model(t)
            map = {attribute: col}
            column = gtk.TreeViewColumn(name, renderer, **map)
            self.data_view.append_column(column)

            # make the renderer editable
            renderer_expert.make_renderer_editable(t, renderer,
                                                   self.model.get_model(), col)

    def set_gadget(self, gadget, proxy):
        super(ModelEditorDialog, self).set_gadget(gadget, proxy)

        model = self.gadget.widget.get_model()
        if model:
            # Get the gazpacho model associated to this gadget
            gzmodel = GazpachoModel.from_model(model)
        else:
            # Or create a new one default one if it doesn't exist
            gzmodel = gadget.project.model_manager.create_model()
        self.model = gzmodel

        # setup the types tab
        self._fill_widget_combo(self.gadget.project)
        self._select_widget_in_widget_combo(self.gadget.name)
        self._synchronize_types_view()

        # do we have hierarchical data?
        self.tree_store_check.set_active(self.model.get_hierarchical())

        # prepare the treeview for data edition
        self._synchronize_data_view()

        # enable or disable buttons depending on the state
        self._update_data_buttons(self.data_view.get_selection())

    # Callbacks here:
    def _on_creation_type_toggled(self, btn):
        self.widget_combo.set_sensitive(
            self.create_from_widget_btn.get_active())

        self.custom_model_align.set_sensitive(
            self.custom_model_btn.get_active())

    def _on_data_view_selection_changed(self, selection):
        self._update_data_buttons(selection)

    def _on_add_row(self, btn):
        model = self.model.get_model()
        iter = model.append(None)
        # make the new row editable
        select_iter(self.data_view, iter)
        path = model.get_path(iter)
        column = self.data_view.get_column(0)
        self.data_view.set_cursor(path, column, True)

    def _on_del_row(self, btn):
        model, iter = self.data_view.get_selection().get_selected()
        if iter:
            model.remove(iter)

    def _on_up_row(self, btn):
        self._move_row(-1, self.model.get_model().move_before)

    def _on_down_row(self, btn):
        self._move_row(1, self.model.get_model().move_after)

    def _on_left_row(self, btn):
        """TODO for hierarchical models"""

    def _on_right_row(self, btn):
        """TODO for hierarchical models"""

    # helpers and internal methods:
    def _update_data_buttons(self, selection):
        # first disable all buttons
        self._set_sensitive_all_data_buttons(False)
        cols = self.data_view.get_columns()
        if not cols:
            return

        # if we have one or more columns that means we can add data
        self.add_row_btn.set_sensitive(True)

        model, iter = selection.get_selected()
        if not iter:
            return

        # if there is a row selected we can delete it
        self.del_row_btn.set_sensitive(True)

        size = len(model)
        if size > 1:
            index = model.get_path(iter)[0]

            # up is available if this is not the first one
            if index != 0:
                self.up_row_btn.set_sensitive(True)

            # down is available if this is not the last one
            if index != (size - 1):
                self.down_row_btn.set_sensitive(True)

    def _set_sensitive_all_data_buttons(self, value):
        self.add_row_btn.set_sensitive(value)
        self.del_row_btn.set_sensitive(value)
        self.up_row_btn.set_sensitive(value)
        self.down_row_btn.set_sensitive(value)
        self.left_row_btn.set_sensitive(value)
        self.right_row_btn.set_sensitive(value)

    def _move_row(self, amount, move_function):
        selection = self.data_view.get_selection()
        model, iter = selection.get_selected()
        if iter:
            index = model.get_path(iter)[0]
            index += amount
            move_function(iter, model.get_iter((index,)))
            self._update_data_buttons(selection)



"""Documentation for module notmm.utils."""

from .configparse import loadconf

__all__ = ['configparse', 'cookiestore', 'django_settings', 
    'cutandpaste', 'YAMLDct', 'YAMLFixture', 'markup', 
    'memcache', 'urlmap', 'testlib', 'template', 'pastelib', 'wsgilib', 'log',
    'paginate', 'paginator']
